@@ -32,6 +32,7 @@ from . import braille
 from . import debug
 from . import generator
 from . import messages
+from . import object_properties
 from . import orca_state
 from . import settings
 from . import settings_manager
@@ -134,7 +135,10 @@ class BrailleGenerator(generator.Generator):
         role = args.get('role', obj.getRole())
         verbosityLevel = _settingsManager.getSetting('brailleVerbosityLevel')
 
-        doNotPresent = [pyatspi.ROLE_UNKNOWN]
+        doNotPresent = [pyatspi.ROLE_UNKNOWN,
+                        pyatspi.ROLE_FILLER,
+                        pyatspi.ROLE_EXTENDED,
+                        pyatspi.ROLE_LINK]
 
         # egg-list-box, e.g. privacy panel in gnome-control-center
         if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
@@ -143,15 +147,16 @@ class BrailleGenerator(generator.Generator):
         if verbosityLevel == settings.VERBOSITY_LEVEL_BRIEF:
             doNotPresent.extend([pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS])
 
-        if (role in _settingsManager.getSetting('brailleForceRoles'))\
-           or (verbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE\
-               and not args.get('readingRow', False)\
-               and role not in doNotPresent):
+        if role == pyatspi.ROLE_HEADING:
+            level = self._script.utilities.headingLevel(obj)
+            result.append(object_properties.ROLE_HEADING_LEVEL_BRAILLE % level)
+
+        elif verbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE \
+           and not args.get('readingRow', False) and role not in doNotPresent:
             result.append(self.getLocalizedRoleName(obj, role))
         return result
 
-    @staticmethod
-    def getLocalizedRoleName(obj, role=None):
+    def getLocalizedRoleName(self, obj, role=None):
         """Returns the localized name of the given Accessible object; the name
         is suitable to be brailled.
 
@@ -296,6 +301,9 @@ class BrailleGenerator(generator.Generator):
             if result and parentResult:
                 result.append(braille.Region(" "))
             result.extend(parentResult)
+            if role == pyatspi.ROLE_EMBEDDED:
+                break
+
             parent = parent.parent
         result.reverse()
         return result
@@ -303,19 +311,13 @@ class BrailleGenerator(generator.Generator):
     def _generateFocusedItem(self, obj, **args):
         result = []
         role = args.get('role', obj.getRole())
-        if role != pyatspi.ROLE_LIST:
+        if role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]:
             return result
 
-        try:
-            s = obj.querySelection()
-        except NotImplementedError:
-            isFocused = \
-                lambda x: x and x.getState().contains(pyatspi.STATE_FOCUSED)
-            items = pyatspi.utils.findAllDescendants(obj, isFocused)
-        else:
-            items = [s.getSelectedChild(i) for i in range(s.nSelectedChildren)]
-            if not items and obj.childCount:
-                items.append(obj[0])
+        items = self._script.utilities.selectedChildren(obj)
+        items = items or [self._script.utilities.focusedChild(obj)]
+        if not (items and items[0]):
+            return result
 
         items = list(map(self._generateName, items))
         for item in items:
