@@ -35,8 +35,6 @@ import pyatspi
 import subprocess
 import sys
 
-from . import orca_state
-
 # Used to turn off all debugging.
 #
 LEVEL_OFF = 10000
@@ -118,6 +116,12 @@ debugFile = None
 eventDebugLevel  = LEVEL_FINEST
 eventDebugFilter = None
 
+# If True, we output debug information for the event queue.  We
+# use this in addition to log level to prevent debug logic from
+# bogging down event handling.
+#
+debugEventQueue = False
+
 # What module(s) should be traced if traceit is being used. By default
 # we'll just attend to ourself. (And by default, we will not enable
 # traceit.) Note that enabling this functionality will drag your system
@@ -163,6 +167,8 @@ TRACE_ROLES = []
 #
 TRACE_ONLY_PROCESSING_EVENTS = True
 
+objEvent = None
+
 def printException(level):
     """Prints out information regarding the current exception.
 
@@ -199,18 +205,25 @@ def println(level, text = ""):
     """
 
     if level >= debugLevel:
+        text = text.replace("\ufffc", "[OBJ]")
         if debugFile:
             try:
                 debugFile.writelines([text, "\n"])
             except TypeError:
                 text = "TypeError when trying to write text"
                 debugFile.writelines([text, "\n"])
+            except:
+                text = "Exception when trying to write text"
+                debugFile.writelines([text, "\n"])
         else:
             try:
                 sys.stderr.writelines([text, "\n"])
             except TypeError:
                 text = "TypeError when trying to write text"
                 sys.stderr.writelines([text, "\n"])
+            except:
+                text = "Exception when trying to write text"
+                sys.stderr.writelines([text, "\n"])
 
 def printResult(level, result=None):
     """Prints the return result, along with information about the
@@ -299,14 +312,19 @@ def getAccessibleDetails(level, acc, indent="", includeApp=True):
         return ""
 
     if includeApp:
-        app = acc.getApplication()
-        if app:
-            try:
-                string = indent + "app.name='%s' " % app.name
-            except (LookupError, RuntimeError):
-                string = indent + "app.name='<error getting name>' "
+        try:
+            app = acc.getApplication()
+        except:
+            string = indent + "app=(exception getting app) "
+            app = None
         else:
-            string = indent + "app=None "
+            if app:
+                try:
+                    string = indent + "app.name='%s' " % app.name
+                except (LookupError, RuntimeError):
+                    string = indent + "app.name='(exception getting name)' "
+            else:
+                string = indent + "app=None "
     else:
         string = indent
 
@@ -341,9 +359,14 @@ def getAccessibleDetails(level, acc, indent="", includeApp=True):
         rel_string = ''
 
     try:
-        string += "name='%s' role='%s' state='%s' relations='%s'" \
+        iface_string = " ".join(pyatspi.utils.listInterfaces(acc))
+    except:
+        iface_string = "(exception calling listInterfaces)"
+
+    try:
+        string += "name='%s' role='%s' state='%s' relations='%s' interfaces='%s'" \
                   % (acc.name or 'None', acc.getRoleName(),
-                     state_string, rel_string)
+                     state_string, rel_string, iface_string)
     except:
         string += "(exception fetching data)"
 
@@ -371,7 +394,6 @@ def _getFileAndModule(frame):
     return filename, module
 
 def _shouldTraceIt():
-    objEvent = orca_state.currentObjectEvent
     if not objEvent:
         return not TRACE_ONLY_PROCESSING_EVENTS
 
