@@ -41,16 +41,25 @@ _scriptManager = script_manager.getManager()
 
 class EventManager:
 
-    def __init__(self):
+    EMBEDDED_OBJECT_CHARACTER = '\ufffc'
+
+    def __init__(self, asyncMode=True):
         debug.println(debug.LEVEL_FINEST, 'INFO: Initializing event manager')
+        debug.println(debug.LEVEL_FINEST, 'INFO: Async Mode is %s' % asyncMode)
+        self._asyncMode = asyncMode
         self._scriptListenerCounts = {}
         self.registry = pyatspi.Registry
+        self._active = False
         self._enqueueCount = 0
         self._dequeueCount = 0
         self._eventQueue     = queue.Queue(0)
         self._gidleId        = 0
         self._gidleLock      = threading.Lock()
-        self.noFocusTimestamp = 0.0
+        self._gilSleepTime = 0.00001
+        self._synchronousToolkits = ['VCL']
+        self._ignoredEvents = ['object:bounds-changed',
+                               'object:state-changed:defunct',
+                               'object:property-change:accessible-parent']
         debug.println(debug.LEVEL_FINEST, 'INFO: Event manager initialized')
 
     def activate(self):
@@ -61,36 +70,135 @@ class EventManager:
         self._registerListener("window:deactivate")
         self._registerListener("object:children-changed")
         self._registerListener("mouse:button")
+        self._active = True
         debug.println(debug.LEVEL_FINEST, 'INFO: Event manager activated')
 
     def deactivate(self):
         """Called when this event manager is deactivated."""
 
         debug.println(debug.LEVEL_FINEST, 'INFO: Dectivating event manager')
+        self._active = False
         for eventType in list(self._scriptListenerCounts.keys()):
             self.registry.deregisterEventListener(self._enqueue, eventType)
         self._scriptListenerCounts = {}
         debug.println(debug.LEVEL_FINEST, 'INFO: Event manager deactivated')
 
+    def ignoreEventTypes(self, eventTypeList):
+        for eventType in eventTypeList:
+            if not eventType in self._ignoredEvents:
+                self._ignoredEvents.append(eventType)
+
+    def unignoreEventTypes(self, eventTypeList):
+        for eventType in eventTypeList:
+            if eventType in self._ignoredEvents:
+                self._ignoredEvents.remove(eventType)
+
     def _ignore(self, event):
         """Returns True if this event should be ignored."""
 
-        ignoredList = ['object:state-changed:defunct',
-                       'object:property-change:accessible-parent']
-        ignoredList.extend(settings.ignoredEventsList)
-        if list(filter(event.type.startswith, ignoredList)):
+        msg = '\nINFO: %s for %s in %s' % (event.type, event.source, event.host_application)
+        debug.println(debug.LEVEL_INFO, msg)
+
+        if not self._active:
+            msg = 'INFO: Ignoring because event manager is not active'
+            debug.println(debug.LEVEL_INFO, msg)
             return True
 
+        if list(filter(event.type.startswith, self._ignoredEvents)):
+            msg = 'INFO: Ignoring because event type is ignored'
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        if event.type.startswith('window'):
+            msg = 'INFO: Not ignoring because event type is never ignored'
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
         # This should ultimately be changed as there are valid reasons
         # to handle these events at the application level.
         if event.type.startswith('object:children-changed:remove') \
            and event.source != self.registry.getDesktop(0):
+            msg = 'INFO: Ignoring because event type is ignored'
+            debug.println(debug.LEVEL_INFO, msg)
             return True
 
+        if event.type.startswith('object:text-changed') and event.type.endswith('system'):
+            # We should also get children-changed events telling us the same thing.
+            # Getting a bunch of both can result in a flood that grinds us to a halt.
+            if event.any_data == self.EMBEDDED_OBJECT_CHARACTER:
+                msg = 'INFO: Text changed event for embedded object. Who cares?'
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
+        try:
+            # TODO - JD: For now we won't ask for the name. Simply asking for the name should
+            # not break anything, and should be a reliable way to quickly identify defunct
+            # objects. But apparently the mere act of asking for the name causes Orca to stop
+            # presenting Eclipse (and possibly other) applications. This might be an AT-SPI2
+            # issue, but until we know for certain....
+            #name = event.source.name
+            state = event.source.getState()
+        except:
+            msg = 'ERROR: %s from potentially-defunct source %s in app %s (%s, %s, %s)' % \
+                  (event.type, event.source, event.host_application, event.detail1,
+                   event.detail2, event.any_data)
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+        if state.contains(pyatspi.STATE_DEFUNCT):
+            msg = 'ERROR: %s from defunct source %s in app %s (%s, %s, %s)' % \
+                  (event.type, event.source, event.host_application, event.detail1,
+                   event.detail2, event.any_data)
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        if event.type.startswith('object:state-changed:showing'):
+            try:
+                role = event.source.getRole()
+            except:
+                role = None
+            if role in [pyatspi.ROLE_IMAGE, pyatspi.ROLE_MENU_ITEM, pyatspi.ROLE_PARAGRAPH]:
+                msg = 'INFO: %s for %s in app %s. Who cares?' % \
+                      (event.type, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
+        if event.type.startswith('object:children-changed:add'):
+            if not event.any_data:
+                msg = 'ERROR: %s without child from source %s in app %s' % \
+                      (event.type, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+            try:
+                state = event.any_data.getState()
+                role = event.any_data.getRole()
+            except:
+                msg = 'ERROR: %s with potentially-defunct child %s from source %s in app %s' % \
+                      (event.type, event.any_data, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+            if state.contains(pyatspi.STATE_DEFUNCT):
+                msg = 'ERROR: %s with defunct child %s from source %s in app %s' % \
+                      (event.type, event.any_data, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
+            # This should be safe. We do not have a reason to present a newly-added,
+            # but not focused image. We do not need to update live regions for images.
+            # This is very likely a completely and utterly useless event for us. The
+            # reason for ignoring it here rather than quickly processing it is the
+            # potential for event floods like we're seeing from matrix.org.
+            if role == pyatspi.ROLE_IMAGE:
+                msg = 'INFO: %s for child image %s from source %s in app %s. Who cares?' % \
+                      (event.type, event.any_data, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
+        msg = 'INFO: Not ignoring due to lack of cause'
+        debug.println(debug.LEVEL_INFO, msg)
         return False
 
     def _addToQueue(self, event, asyncMode):
-        debugging = settings.debugEventQueue
+        debugging = debug.debugEventQueue
         if debugging:
             debug.println(debug.LEVEL_ALL, "           acquiring lock...")
         self._gidleLock.acquire()
@@ -106,14 +214,14 @@ class EventManager:
             debug.println(debug.LEVEL_ALL, "           ...put complete")
 
         if asyncMode and not self._gidleId:
-            if settings.gilSleepTime:
-                time.sleep(settings.gilSleepTime)
+            if self._gilSleepTime:
+                time.sleep(self._gilSleepTime)
             self._gidleId = GLib.idle_add(self._dequeue)
 
         if debugging:
             debug.println(debug.LEVEL_ALL, "           releasing lock...")
         self._gidleLock.release()
-        if settings.debugEventQueue:
+        if debug.debugEventQueue:
             debug.println(debug.LEVEL_ALL, "           ...released")
 
     def _queuePrintln(self, e, isEnqueue=True):
@@ -124,15 +232,16 @@ class EventManager:
         elif isinstance(e, input_event.BrailleEvent):
             data = "'%s'" % repr(e.event)
         elif not debug.eventDebugFilter or debug.eventDebugFilter.match(e.type):
-            data = ""
+            data = "%s (%s,%s,%s) from %s" % \
+                   (e.source, e.detail1, e.detail2, e.any_data, e.host_application)
         else:
             return
 
         eType = str(e.type).upper()
         if isEnqueue:
-            string = "----------> QUEUEING %s %s" % (eType, data)
+            string = "----------> QUEUEING %s %s" % (eType, data.upper())
         else:
-            string = "DEQUEUED %s %s <----------" % (eType, data)
+            string = "DEQUEUED %s %s <----------" % (eType, data.upper())
 
         debug.println(debug.LEVEL_ALL, string)
 
@@ -143,7 +252,7 @@ class EventManager:
         - e: an at-spi event.
         """
 
-        if settings.debugEventQueue:
+        if debug.debugEventQueue:
             if self._enqueueCount:
                 msg = "_enqueue entered before exiting (count = %d)" \
                     % self._enqueueCount
@@ -152,21 +261,28 @@ class EventManager:
 
         inputEvents = (input_event.KeyboardEvent, input_event.BrailleEvent)
         isObjectEvent = not isinstance(e, inputEvents)
-        if isObjectEvent and self._ignore(e):
-            if settings.debugEventQueue:
+
+        try:
+            ignore = isObjectEvent and self._ignore(e)
+        except:
+            msg = 'ERROR: Exception evaluating event: %s' % e
+            debug.println(debug.LEVEL_INFO, msg)
+            ignore = True
+        if ignore:
+            if debug.debugEventQueue:
                 self._enqueueCount -= 1
             return
 
         self._queuePrintln(e)
 
-        asyncMode = settings.asyncMode
+        asyncMode = self._asyncMode
         if isObjectEvent:
             app = e.source.getApplication()
             try:
                 toolkitName = app.toolkitName
             except:
                 toolkitName = None
-            if toolkitName in settings.synchronousToolkits:
+            if toolkitName in self._synchronousToolkits:
                 asyncMode = False
             script = _scriptManager.getScript(app, e.source)
             script.eventCache[e.type] = (e, time.time())
@@ -175,7 +291,7 @@ class EventManager:
         if not asyncMode:
             self._dequeue()
 
-        if settings.debugEventQueue:
+        if debug.debugEventQueue:
             self._enqueueCount -= 1
 
     def _dequeue(self):
@@ -184,7 +300,7 @@ class EventManager:
 
         rerun = True
 
-        if settings.debugEventQueue:
+        if debug.debugEventQueue:
             debug.println(debug.LEVEL_ALL,
                           "event_manager._dequeue %d" % self._dequeueCount)
             self._dequeueCount += 1
@@ -196,7 +312,7 @@ class EventManager:
             if isinstance(event, inputEvents):
                 self._processInputEvent(event)
             else:
-                orca_state.currentObjectEvent = event
+                debug.objEvent = event
                 debugging = not debug.eventDebugFilter \
                             or debug.eventDebugFilter.match(event.type)
                 if debugging:
@@ -212,7 +328,7 @@ class EventManager:
                     debug.println(debug.eventDebugLevel,
                                   "^^^^^ PROCESS OBJECT EVENT %s ^^^^^\n" \
                                   % event.type)
-                orca_state.currentObjectEvent = None
+                debug.objEvent = None
 
             # [[[TODO: HACK - it would seem logical to only do this if we
             # discover the queue is empty, but this inroduces a hang for
@@ -220,22 +336,18 @@ class EventManager:
             # lock.  So...we do it here.]]]
             #
             try:
-                noFocus = \
-                    not orca_state.activeScript \
-                    or (not orca_state.locusOfFocus and \
-                        self.noFocusTimestamp != orca_state.noFocusTimestamp)
+                noFocus = not (orca_state.activeScript or orca_state.locusOfFocus)
             except:
                 noFocus = True
 
             self._gidleLock.acquire()
             if self._eventQueue.empty():
                 if noFocus:
-                    if settings.gilSleepTime:
-                        time.sleep(settings.gilSleepTime)
+                    if self._gilSleepTime:
+                        time.sleep(self._gilSleepTime)
                     fullMessage = messages.NO_FOCUS
                     defaultScript = _scriptManager.getDefaultScript()
                     defaultScript.presentMessage(fullMessage, '')
-                    self.noFocusTimestamp = orca_state.noFocusTimestamp
                 self._gidleId = 0
                 rerun = False # destroy and don't call again
             self._gidleLock.release()
@@ -247,7 +359,7 @@ class EventManager:
         except:
             debug.printException(debug.LEVEL_SEVERE)
 
-        if settings.debugEventQueue:
+        if debug.debugEventQueue:
             self._dequeueCount -= 1
             debug.println(debug.LEVEL_ALL,
                           "Leaving _dequeue. Count: %d" % self._dequeueCount)
@@ -386,6 +498,7 @@ class EventManager:
             return
 
         eType = str(event.type).upper()
+        startTime = time.time()
         debug.println(debug.eventDebugLevel,
                       "\nvvvvv PROCESS %s %s vvvvv" % (eType, data))
         try:
@@ -393,6 +506,9 @@ class EventManager:
         except:
             debug.printException(debug.LEVEL_WARNING)
             debug.printStack(debug.LEVEL_WARNING)
+        debug.println(debug.eventDebugLevel,
+                      "TOTAL PROCESSING TIME: %.4f" \
+                      % (time.time() - startTime))
         debug.println(debug.eventDebugLevel,
                       "^^^^^ PROCESS %s %s ^^^^^\n" % (eType, data))
 
@@ -404,6 +520,7 @@ class EventManager:
             return orca_state.activeScript
 
         script = None
+        app = None
         try:
             app = event.host_application or event.source.getApplication()
             if app and app.getState().contains(pyatspi.STATE_DEFUNCT):
@@ -414,8 +531,12 @@ class EventManager:
             msg = 'WARNING: Exception when getting script for event.'
             debug.println(debug.LEVEL_WARNING, msg)
         else:
+            msg = 'INFO: Getting script for %s from %s' % (event.type, app)
+            debug.println(debug.LEVEL_INFO, msg)
             script = _scriptManager.getScript(app, event.source)
 
+        msg = 'INFO: Script for %s from %s is %s' % (event.type, app, script)
+        debug.println(debug.LEVEL_INFO, msg)
         return script
 
     def _isActivatableEvent(self, event, script=None):
@@ -531,15 +652,30 @@ class EventManager:
 
         script = self._getScriptForEvent(event)
         if not script:
+            msg = 'ERROR: Could not get script for %s' % event
+            debug.println(debug.LEVEL_INFO, msg)
             return
 
-        debug.println(debug.LEVEL_FINEST, "Script for event: %s" % script.name)
         setNewActiveScript, reason = self._isActivatableEvent(event, script)
         if setNewActiveScript:
-            app = event.host_application or event.source.getApplication()
-            _scriptManager.setActiveScript(script, reason)
+            try:
+                app = event.host_application or event.source.getApplication()
+            except:
+                msg = 'ERROR: Could not get application for %s' % event.source
+                debug.println(debug.LEVEL_INFO, msg)
+                return
+            try:
+                _scriptManager.setActiveScript(script, reason)
+            except:
+                msg = 'ERROR: Could not set active script for %s' % event.source
+                debug.println(debug.LEVEL_INFO, msg)
+                return
 
-        script.processObjectEvent(event)
+        try:
+            script.processObjectEvent(event)
+        except:
+            msg = 'ERROR: Could not process %s' % event.type
+            debug.println(debug.LEVEL_INFO, msg)
 
     def processKeyboardEvent(self, keyboardEvent):
         """Processes the given keyboard event based on the keybinding from the
