@@ -36,9 +36,6 @@ from . import object_properties
 from . import orca_state
 from . import settings
 
-from .braille_generator import BrailleGenerator
-from .orca_i18n import _
-
 # [[[WDW - HACK Regular expression to split strings on whitespace
 # boundaries, which is what we'll use for word dividers instead of
 # living at the whim of whomever decided to implement the AT-SPI
@@ -428,7 +425,8 @@ class ValueZone(Zone):
 
             speechValue = speechValue + " " + messages.percentage(percentValue)
 
-            rolename = BrailleGenerator.getLocalizedRoleName(self.accessible)
+            rolename = orca_state.activeScript.brailleGenerator.getLocalizedRoleName(
+                self.accessible)
             if orientation:
                 brailleValue = "%s %s %d%%" % (orientation,
                     rolename,
@@ -594,12 +592,7 @@ class Context:
         else:
             # We want to stop at the window or frame or equivalent level.
             #
-            obj = orca_state.locusOfFocus
-            while obj \
-                      and obj.parent \
-                      and (obj.parent.getRole() != pyatspi.ROLE_APPLICATION) \
-                      and (obj != obj.parent):
-                obj = obj.parent
+            obj = script.utilities.topLevelObject(orca_state.locusOfFocus)
             if obj:
                 self.lines = self.clusterZonesByLine(self.getShowingZones(obj))
             else:
@@ -616,7 +609,7 @@ class Context:
             role = None
 
         if role == pyatspi.ROLE_TABLE_CELL:
-            searchZone = orca_state.activeScript.\
+            searchZone = self.script.\
                 utilities.realActiveDescendant(orca_state.locusOfFocus)
         else:
             searchZone = orca_state.locusOfFocus
@@ -774,7 +767,7 @@ class Context:
             else:
                 [x, y, width, height] = text.getRangeExtents(
                         substringStartOffset, substringEndOffset, 0)
-                if self.script.utilities.isVisibleRegion(
+                if self.script.utilities.containsRegion(
                         x, y, width, height,
                         cliprect.x, cliprect.y,
                         cliprect.width, cliprect.height):
@@ -1034,25 +1027,8 @@ class Context:
 
         elif role == pyatspi.ROLE_TABLE_CELL:
             # Handle table cells that act like check boxes.
-            #
-            try:
-                action = accessible.queryAction()
-            except NotImplementedError:
-                action = None
-                
-            if action:
-                hasToggle = False
-                for i in range(0, action.nActions):
-                    # Translators: this is the action name for
-                    # the 'toggle' action. It must be the same
-                    # string used in the *.po file for gail.
-                    #
-                    if action.getName(i) in ["toggle", _("toggle")]:
-                        hasToggle = True
-                        break
-                if hasToggle:
-                    self._insertStateZone(zones, accessible, 
-                                          pyatspi.ROLE_CHECK_BOX)
+            if self.script.utilities.hasMeaningfulToggleAction(accessible):
+                self._insertStateZone(zones, accessible, pyatspi.ROLE_CHECK_BOX)
 
         if zone:
             if stateOnLeft:
@@ -1075,7 +1051,7 @@ class Context:
         #
         extents = icomponent.getExtents(0)
 
-        if not self.script.utilities.isVisibleRegion(
+        if not self.script.utilities.containsRegion(
                 extents.x, extents.y,
                 extents.width, extents.height,
                 cliprect.x, cliprect.y,
@@ -1125,7 +1101,7 @@ class Context:
             [width, height] = iimage.getImageSize()
 
             if width != 0 and height != 0 \
-               and self.script.utilities.isVisibleRegion(
+               and self.script.utilities.containsRegion(
                     x, y, width, height,
                     cliprect.x, cliprect.y,
                     cliprect.width, cliprect.height):
