@@ -155,6 +155,9 @@ class Generator:
         else:
             del args['role']
 
+    def generateContents(self, contents, **args):
+        return []
+
     def generate(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) that represent the complete presentatin for the
@@ -228,7 +231,7 @@ class Generator:
             debug.println(debug.LEVEL_ALL, "\nPREPARATION TIME: %s" % duration)
             debug.println(
                 debug.LEVEL_ALL,
-                "generate %s for %s %s (args=%s) using '%s'" \
+                "generate %s for %s %s\n(args=%s)\nusing '%s'" \
                 % (self._mode,
                    args['formatType'], 
                    details,
@@ -323,6 +326,16 @@ class Generator:
                 return result
             if description:
                 result.append(description)
+            else:
+                link = None
+                if obj.getRole() == pyatspi.ROLE_LINK:
+                    link = obj
+                elif obj.parent.getRole() == pyatspi.ROLE_LINK:
+                    link = obj.parent
+                if link:
+                    basename = self._script.utilities.linkBasename(link)
+                    if basename:
+                        result.append(basename)
         # To make the unlabeled icons in gnome-panel more accessible.
         try:
             role = args.get('role', obj.getRole())
@@ -378,9 +391,12 @@ class Generator:
         is different from that of the name and label.
         """
         result = []
-        label = self._script.utilities.displayedLabel(obj)
-        if obj.description and not obj.description in [obj.name, label]:
-            result.append(obj.description)
+        if obj.description:
+            label = self._script.utilities.displayedLabel(obj) or ""
+            name = obj.name or ""
+            desc = obj.description.lower()
+            if not (desc in name.lower() or desc in label.lower()):
+                result.append(obj.description)
         return result
 
     def _generateLabel(self, obj, **args):
@@ -424,23 +440,9 @@ class Generator:
     #####################################################################
 
     def _generateClickable(self, obj, **args):
-        if not args.get('mode', None):
-            args['mode'] = self._mode
-        args['stringType'] = 'clickable'
-
-        if self._script.utilities.isClickableElement(obj):
-            return [self._script.formatting.getString(**args)]
-
         return []
 
     def _generateHasLongDesc(self, obj, **args):
-        if not args.get('mode', None):
-            args['mode'] = self._mode
-        args['stringType'] = 'haslongdesc'
-
-        if self._script.utilities.hasLongDesc(obj):
-            return [self._script.formatting.getString(**args)]
-
         return []
 
     def _generateAvailability(self, obj, **args):
@@ -766,10 +768,14 @@ class Generator:
         """Returns an array of strings for use by speech and braille to present
         the size of a table."""
 
+        if self._script.utilities.isLayoutOnly(obj):
+            return []
+
         try:
             table = obj.queryTable()
         except:
             return []
+
         return [messages.tableSize(table.nRows, table.nColumns)]       
 
     def _generateTableCellRow(self, obj, **args):
@@ -851,7 +857,7 @@ class Generator:
     def _generateSubstring(self, obj, **args):
         start = args.get('startOffset')
         end = args.get('endOffset')
-        if start == None or end == None:
+        if start is None or end is None:
             return []
 
         substring = self._script.utilities.substring(obj, start, end)
@@ -954,6 +960,11 @@ class Generator:
         """Returns an array of strings for use by speech and braille that
         represent the nesting level of an object in a list.
         """
+        start = args.get('startOffset')
+        end = args.get('endOffset')
+        if start is not None and end is not None:
+            return []
+
         result = []
         if not args.get('mode', None):
             args['mode'] = self._mode
@@ -1033,3 +1044,6 @@ class Generator:
                     break
             parent = parent.parent
         return result
+
+    def _generatePageSummary(self, obj, **args):
+        return []
