@@ -39,6 +39,42 @@ from .orca_i18n import _, C_
 # it to show all of its contents. And so on.
 ACTIVATE = _("_Activate")
 
+# Translators: Orca has a number of commands that override the default behavior
+# within an application. For instance, on a web page Orca's Structural Navigation
+# command "h" moves you to the next heading. What should happen when you press
+# "h" in an entry on a web page depends: If you want to resume reading content,
+# "h" should move to the next heading; if you want to enter text, "h" should not
+# move you to the next heading. Because Orca doesn't know what you want to do,
+# it has two modes: In browse mode, Orca treats key presses as commands to read
+# the content; in focus mode, Orca treats key presses as something that should be
+# handled by the focused widget. Orca optionally can attempt to detect which mode
+# is appropriate for the current situation and switch automatically. This string
+# is a label for a GUI option to enable such automatic switching when structural
+# navigation commands are used. As an example, if this setting were enabled,
+# pressing "e" to move to the next entry would move focus there and also turn
+# focus mode on so that the next press of "e" would type an "e" into the entry.
+# If this setting is not enabled, the second press of "e" would continue to be
+# a navigation command to move amongst entries.
+AUTO_FOCUS_MODE_STRUCT_NAV = _("Automatic focus mode during structural navigation")
+
+# Translators: Orca has a number of commands that override the default behavior
+# within an application. For instance, if you are at the bottom of an entry and
+# press Down arrow, should you leave the entry? It depends on if you want to
+# resume reading content or if you are editing the text in the entry. Because
+# Orca doesn't know what you want to do, it has two modes: In browse mode, Orca
+# treats key presses as commands to read the content; in focus mode, Orca treats
+# key presses as something that should be handled by the focused widget. Orca
+# optionally can attempt to detect which mode is appropriate for the current
+# situation and switch automatically. This string is a label for a GUI option to
+# enable such automatic switching when caret navigation commands are used. As an
+# example, if this setting were enabled, pressing Down Arrow would allow you to
+# move into an entry but once you had done so, Orca would switch to Focus mode
+# and subsequent presses of Down Arrow would be controlled by the web browser
+# and not by Orca. If this setting is not enabled, Orca would continue to control
+# what happens when you press an arrow key, thus making it possible to arrow out
+# of the entry.
+AUTO_FOCUS_MODE_CARET_NAV = _("Automatic focus mode during caret navigation")
+
 # Translators: A single braille cell on a refreshable braille display consists
 # of 8 dots. Dot 7 is the dot in the bottom left corner. If the user selects 
 # this option, Dot 7 will be used to 'underline' text of interest, e.g. when
@@ -66,23 +102,6 @@ BTN_JUMP_TO = _("_Jump to")
 # Translators: This is the label for a button in a dialog.
 BTN_OK = _("_OK")
 
-# Translators: Orca has had to implement its own caret navigation model to work
-# around issues in Gecko/Firefox. In some versions of Firefox, we must perform
-# a focus grab on each object being navigated in order for things to work as
-# expected; in other versions of Firefox, we must avoid doing so in order for
-# things# to work as expected. We cannot identify with certainty which situation
-# the user is in, so we must provide this as an option within Orca.
-CARET_NAVIGATION_GRAB_FOCUS = _("_Grab focus on objects when navigating")
-
-# Translators: When the user arrows up and down in HTML content, and Orca is
-# controlling the caret, the user might want Orca to always position the
-# cursor at the beginning of the line (as opposed to the position directly
-# above/below the current cursor position). Different users have different
-# preferences. This string is the label for a checkbox which allows users
-# to set the line-positioning behavior they want.
-CARET_NAVIGATION_START_OF_LINE = \
-    _("_Position cursor at start of line when navigating vertically")
-
 # Translators: If this checkbox is checked, then Orca will tell you when one of
 # your buddies is typing a message.
 CHAT_ANNOUNCE_BUDDY_TYPING = _("Announce when your _buddies are typing")
@@ -106,7 +125,7 @@ CHAT_SPEAK_MESSAGES_ALL = _("All cha_nnels")
 
 # Translators: This is the label of a radio button. If it is selected, Orca will
 # speak all new chat messages as they appear if and only if the chat application
-# has focus. The string substituion is for the application name (e.g Pidgin).
+# has focus. The string substitution is for the application name (e.g Pidgin).
 CHAT_SPEAK_MESSAGES_ALL_IF_FOCUSED = _("All channels when an_y %s window is active")
 
 # Translators: This is the label of a radio button. If it is selected, Orca will
@@ -118,6 +137,18 @@ CHAT_SPEAK_MESSAGES_ACTIVE = _("A channel only if its _window is active")
 # chat room prior to presenting an incoming message.
 CHAT_SPEAK_ROOM_NAME = _("_Speak Chat Room name")
 
+# Translators: When presenting the content of a line on a web page, Orca by
+# default presents the full line, including any links or form fields on that
+# line, in order to reflect the on-screen layout as seen by sighted users.
+# Not all users like this presentation, however, and prefer to have objects
+# treated as if they were on individual lines, such as is done by Windows
+# screen readers, so that unrelated objects (e.g. links in a navbar) are not
+# all jumbled together. As a result, this is now configurable. If layout mode
+# is enabled, Orca will present the full line as it appears on the screen; if
+# it is disabled, Orca will treat each object as if it were on a separate line,
+# both for presentation and navigation.
+CONTENT_LAYOUT_MODE = _("Enable layout mode for content")
+
 # Translators: Orca's keybindings support double and triple "clicks" or key
 # presses, similar to using a mouse. This string appears in Orca's preferences
 # dialog after a keybinding which requires a double click.
@@ -377,6 +408,12 @@ SN_HEADER_CAPTION = C_("structural navigation", "Caption")
 # contains the label of a check box.
 SN_HEADER_CHECK_BOX = C_("structural navigation", "Check Box")
 
+# Translators: Orca has a command that presents a list of structural navigation
+# objects in a dialog box so that users can navigate more quickly than they
+# could with native keyboard navigation. This is the title for a column which
+# contains the text displayed for a web element with an "onClick" handler.
+SN_HEADER_CLICKABLE = C_("structural navigation", "Clickable")
+
 # Translators: Orca has a command that presents a list of structural navigation
 # objects in a dialog box so that users can navigate more quickly than they
 # could with native keyboard navigation. This is the title for a column which
@@ -395,12 +432,26 @@ SN_HEADER_DESCRIPTION = C_("structural navigation", "Description")
 # contains the text of a heading.
 SN_HEADER_HEADING = C_("structural navigation", "Heading")
 
+# Translators: Orca has a command that presents a list of structural navigation
+# objects in a dialog box so that users can navigate more quickly than they
+# could with native keyboard navigation. This is the title for a column which
+# contains the text (alt text, title, etc.) associated with an image.
+SN_HEADER_IMAGE = C_("structural navigation", "Image")
+
 # Translators: Orca has a command that presents a list of structural navigation
 # objects in a dialog box so that users can navigate more quickly than they
 # could with native keyboard navigation. This is the title for a column which
 # contains the label of a form field.
 SN_HEADER_LABEL = C_("structural navigation", "Label")
 
+# Translators: Orca has a command that presents a list of structural navigation
+# objects in a dialog box so that users can navigate more quickly than they
+# could with native keyboard navigation. This is the title for a column which
+# contains the text of a landmark. ARIA role landmarks are the W3C defined HTML
+# tag attribute 'role' used to identify important part of webpage like banners,
+# main context, search etc.
+SN_HEADER_LANDMARK = C_("structural navigation", "Landmark")
+
 # Translators: Orca has a command that presents a list of structural navigation
 # objects in a dialog box so that users can navigate more quickly than they
 # could with native keyboard navigation. This is the title of a column which
@@ -455,7 +506,7 @@ SN_HEADER_ROLE = C_("structural navigation", "Role")
 # objects in a dialog box so that users can navigate more quickly than they
 # could with native keyboard navigation. This is the title for a column which
 # contains the selected item of a form field.
-SN_HEADER_SELETED_ITEM = C_("structural navigation", "Selected Item")
+SN_HEADER_SELECTED_ITEM = C_("structural navigation", "Selected Item")
 
 # Translators: Orca has a command that presents a list of structural navigation
 # objects in a dialog box so that users can navigate more quickly than they
@@ -497,6 +548,12 @@ SN_TITLE_BUTTON = C_("structural navigation", "Buttons")
 # could with native keyboard navigation. This is the title of such a dialog box.
 SN_TITLE_CHECK_BOX = C_("structural navigation", "Check Boxes")
 
+# Translators: Orca has a command that presents a list of structural navigation
+# objects in a dialog box so that users can navigate more quickly than they
+# could with native keyboard navigation. This is the title of such a dialog box.
+# "Clickables" are web elements which have an "onClick" handler.
+SN_TITLE_CLICKABLE = C_("structural navigation", "Clickables")
+
 # Translators: Orca has a command that presents a list of structural navigation
 # objects in a dialog box so that users can navigate more quickly than they
 # could with native keyboard navigation. This is the title of such a dialog box.
@@ -517,12 +574,24 @@ SN_TITLE_FORM_FIELD = C_("structural navigation", "Form Fields")
 # could with native keyboard navigation. This is the title of such a dialog box.
 SN_TITLE_HEADING = C_("structural navigation", "Headings")
 
+# Translators: Orca has a command that presents a list of structural navigation
+# objects in a dialog box so that users can navigate more quickly than they
+# could with native keyboard navigation. This is the title of such a dialog box.
+SN_TITLE_IMAGE = C_("structural navigation", "Images")
+
 # Translators: Orca has a command that presents a list of structural navigation
 # objects in a dialog box so that users can navigate more quickly than they
 # could with native keyboard navigation. This is the title of such a dialog box.
 # Level will be a "1" for <h1>, a "2" for <h2>, and so on.
 SN_TITLE_HEADING_AT_LEVEL = C_("structural navigation", "Headings at Level %d")
 
+# Translators: Orca has a command that presents a list of structural navigation
+# objects in a dialog box so that users can navigate more quickly than they
+# could with native keyboard navigation. This is the title of such a dialog box.
+# ARIA role landmarks are the W3C defined HTML tag attribute 'role' used to
+# identify important part of webpage like banners, main context, search etc.
+SN_TITLE_LANDMARK = C_("structural navigation", "Landmarks")
+
 # Translators: Orca has a command that presents a list of structural navigation
 # objects in a dialog box so that users can navigate more quickly than they
 # could with native keyboard navigation. This is the title of such a dialog box.
@@ -584,7 +653,7 @@ READ_PAGE_UPON_LOAD = \
 # Translators: Different speech systems and speech engines work differently when
 # it comes to handling pauses (e.g. sentence boundaries). This property allows
 # the user to specify whether speech should be sent to the speech synthesis
-# system immediately when a pause directive is enountered or if it should be
+# system immediately when a pause directive is encountered or if it should be
 # queued up and sent to the speech synthesis system once the entire set of
 # utterances has been calculated.
 SPEECH_BREAK_INTO_CHUNKS = _("Break speech into ch_unks between pauses")
@@ -621,6 +690,28 @@ SPEECH_VOICE_TYPE_UPPERCASE = C_("VoiceType", "Uppercase")
 # system. (http://devel.freebsoft.org/speechd)
 SPEECH_DISPATCHER = _("Speech Dispatcher")
 
+# Translators: This is a label for a group of options related to Orca's behavior
+# when presenting an application's spell check dialog.
+SPELL_CHECK = C_("OptionGroup", "Spell Check")
+
+# Translators: This is a label for a checkbox associated with an Orca setting.
+# When this option is enabled, Orca will spell out the current error in addition
+# to speaking it. For example, if the misspelled word is "foo," enabling this
+# setting would cause Orca to speak "f o o" after speaking "foo".
+SPELL_CHECK_SPELL_ERROR = _("Spell _error")
+
+# Translators: This is a label for a checkbox associated with an Orca setting.
+# When this option is enabled, Orca will spell out the current suggestion in
+# addition to speaking it. For example, if the misspelled word is "foo," and
+# the first suggestion is "for" enabling this setting would cause Orca to speak
+# "f o r" after speaking "for".
+SPELL_CHECK_SPELL_SUGGESTION = _("Spell _suggestion")
+
+# Translators: This is a label for a checkbox associated with an Orca setting.
+# When this option is enabled, Orca will present the context (surrounding text,
+# typically the sentence or line) in which the mistake occurred.
+SPELL_CHECK_PRESENT_CONTEXT = _("Present _context of error")
+
 # Translators: This is a label for an option to tell Orca whether or not it
 # should speak the coordinates of the current spread sheet cell. Coordinates are
 # the row and column position within the spread sheet (i.e. A1, B1, C2 ...)
