@@ -32,10 +32,11 @@ import time
 import unicodedata
 
 from . import debug
+from . import keybindings
 from . import keynames
+from . import messages
 from . import orca_state
 from . import settings
-from .orca_i18n import C_
 
 KEYBOARD_EVENT     = "keyboard"
 BRAILLE_EVENT      = "braille"
@@ -43,6 +44,8 @@ MOUSE_BUTTON_EVENT = "mouse:button"
 
 class InputEvent:
 
+    _clickCount = 0
+
     def __init__(self, eventType):
         """Creates a new input event of the given type.
 
@@ -58,7 +61,7 @@ class InputEvent:
         # TODO - JD: I relocated this out of script.py, because it seems
         # to belong there even less than here. Need to revisit how this
         # functionality is used and where.
-        return orca_state.clickCount
+        return InputEvent._clickCount
 
     def setClickCount(self):
         """Sets the count of the number of clicks a user has made to one
@@ -79,21 +82,21 @@ class InputEvent:
             return
 
         if not isinstance(self, KeyboardEvent):
-            orca_state.clickCount = 0
+            InputEvent._clickCount = 0
             return
 
         if not isinstance(lastInputEvent, KeyboardEvent):
-            orca_state.clickCount = 1
+            InputEvent._clickCount = 1
             return
 
         if self.time - lastInputEvent.time < settings.doubleClickTimeout \
             and lastInputEvent.event_string == self.event_string:
             # Cap the possible number of clicks at 3.
-            if orca_state.clickCount < 3:
-                orca_state.clickCount += 1
+            if InputEvent._clickCount < 3:
+                InputEvent._clickCount += 1
                 return
 
-        orca_state.clickCount = 1
+        InputEvent._clickCount = 1
 
 class KeyboardEvent(InputEvent):
 
@@ -145,7 +148,7 @@ class KeyboardEvent(InputEvent):
         # conversion. [[[WDW - this is making assumptions about
         # mapping ASCII control characters to UTF-8.]]]
         #
-        if (self.modifiers & settings.CTRL_MODIFIER_MASK) \
+        if (self.modifiers & keybindings.CTRL_MODIFIER_MASK) \
             and (not self.is_text) and (len(self.event_string) == 1):
             value = ord(self.event_string[0])
             if value < 32:
@@ -183,6 +186,17 @@ class KeyboardEvent(InputEvent):
         if not self.isLockingKey():
             self.shouldEcho = self.shouldEcho and settings.enableKeyEcho
 
+    def __eq__(self, other):
+        if not other:
+            return False
+
+        if self.type == other.type \
+           and self.hw_code == other.hw_code \
+           and self.timestamp == other.timestamp:
+            return True
+
+        return False
+
     def toString(self):
         return ("KEYBOARDEVENT: type=%d\n" % self.type) \
             + ("                id=%d\n" % self.id) \
@@ -259,7 +273,8 @@ class KeyboardEvent(InputEvent):
 
         return self.event_string in \
             ['Alt_L', 'Alt_R', 'Control_L', 'Control_R',
-             'Shift_L', 'Shift_R', 'Meta_L', 'Meta_R']
+             'Shift_L', 'Shift_R', 'Meta_L', 'Meta_R',
+             'ISO_Level3_Shift']
 
     def isOrcaModifier(self):
         """Return True if this is the Orca modifier key."""
@@ -267,7 +282,15 @@ class KeyboardEvent(InputEvent):
         if orca_state.bypassNextCommand:
             return False
 
-        return self.event_string in settings.orcaModifierKeys
+        if self.event_string in settings.orcaModifierKeys:
+            return True
+
+        if self.keyval_name == "KP_0" \
+           and "KP_Insert" in settings.orcaModifierKeys \
+           and self.modifiers & keybindings.SHIFT_MODIFIER_MASK:
+            return True
+
+        return False
 
     def isOrcaModified(self):
         """Return True if this key is Orca modified."""
@@ -275,7 +298,7 @@ class KeyboardEvent(InputEvent):
         if orca_state.bypassNextCommand:
             return False
 
-        return self.modifiers & settings.ORCA_MODIFIER_MASK
+        return self.modifiers & keybindings.ORCA_MODIFIER_MASK
 
     def isPrintableKey(self):
         """Return True if this is a printable key."""
@@ -339,95 +362,15 @@ class KeyboardEvent(InputEvent):
             return ''
 
         if not locked:
-            # Translators: This string is used to present the state of a
-            # locking key, such as Caps Lock. If Caps Lock is "off", then
-            # letters typed will appear in lowercase; if Caps Lock is "on",
-            # they will instead appear in uppercase. This string is also
-            # applied to Num Lock and potentially will be applied to similar
-            # keys in the future.
-            return C_("locking key state", "off")
+            return messages.LOCKING_KEY_STATE_OFF
 
-        # Translators: This string is used to present the state of a
-        # locking key, such as Caps Lock. If Caps Lock is "off", then
-        # letters typed will appear in lowercase; if Caps Lock is "on",
-        # they will instead appear in uppercase. This string is also
-        # applied to Num Lock and potentially will be applied to similar
-        # keys in the future.
-        return C_("locking key state", "on")
+        return messages.LOCKING_KEY_STATE_ON
 
     def getKeyName(self):
         """Returns the string to be used for presenting the key to the user."""
 
         return keynames.getKeyName(self.event_string)
 
-    def ignoreDueToTimestamp(self):
-        """Returns True if the event should be ignored due to its timestamp,
-        which might be completely absent or suggest that this input event is
-        a duplicate."""
-
-        if not self.timestamp:
-            return True
-        if self.timestamp != orca_state.lastInputEventTimestamp:
-            return False
-        if not orca_state.lastInputEvent:
-            return False
-        if isinstance(self, MouseButtonEvent) \
-           or isinstance(orca_state.lastInputEvent, MouseButtonEvent):
-            return False
-        if self.hw_code == orca_state.lastInputEvent.hw_code \
-           and self.type == orca_state.lastInputEvent.type:
-            return True
-
-        return False
-
-    def present(self):
-        """Presents the event via the appropriate medium/media. Returns True
-        if we presented the event. False if there was some reason the event
-        was not worthy of presentation."""
-
-        if not orca_state.learnModeEnabled:
-            if self.shouldEcho == False or self.isOrcaModified():
-                return False
-
-        try:
-            role = orca_state.locusOfFocus.getRole()
-        except:
-            return False
-
-        if role == pyatspi.ROLE_PASSWORD_TEXT:
-            return False
-
-        # Worst. Hack. EVER. We have no reliable way of knowing a password is
-        # being entered into a terminal -- other than the fact that the text
-        # typed ain't there. As a result, we have to do special things when
-        # not in special modes. :( See bgo 668025.
-        if role == pyatspi.ROLE_TERMINAL:
-            if not self.isPressedKey():
-                try:
-                    text = orca_state.locusOfFocus.queryText()
-                    o = text.caretOffset
-                    string = text.getText(o-1, o)
-                except:
-                    pass
-                else:
-                    if not self.event_string in [string, 'space']:
-                        return False
-            elif not (orca_state.learnModeEnabled or self.isLockingKey()):
-                return False
-
-        elif not self.isPressedKey():
-            return False
-
-        orca_state.lastKeyEchoTime = time.time()
-        debug.println(debug.LEVEL_FINEST,
-                      "KeyboardEvent.present: %s" % self.event_string)
-
-        script = orca_state.activeScript
-        if script:
-            return script.presentKeyboardEvent(self)
-
-        return False
-
 class BrailleEvent(InputEvent):
 
     def __init__(self, event):
