@@ -39,8 +39,6 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import imp
-import importlib
 import pyatspi
 
 from . import braille_generator
@@ -100,13 +98,16 @@ class Script:
         self.utilities = self.getUtilities()
         self.labelInference = self.getLabelInference()
         self.structuralNavigation = self.getStructuralNavigation()
+        self.caretNavigation = self.getCaretNavigation()
+        self.bookmarks = self.getBookmarks()
+        self.liveRegionManager = self.getLiveRegionManager()
+
         self.chat = self.getChat()
         self.inputEventHandlers = {}
         self.pointOfReference = {}
         self.setupInputEventHandlers()
         self.keyBindings = self.getKeyBindings()
         self.brailleBindings = self.getBrailleBindings()
-        self.app_pronunciation_dict = self.getPronunciations()
 
         self.formatting = self.getFormatting()
         self.brailleGenerator = self.getBrailleGenerator()
@@ -114,13 +115,14 @@ class Script:
         self.generatorCache = {}
         self.eventCache = {}
         self.whereAmI = self.getWhereAmI()
-        self.bookmarks = self.getBookmarks()
+        self.spellcheck = self.getSpellCheck()
         self.voices = settings.voices
         self.tutorialGenerator = self.getTutorialGenerator()
 
         self.flatReviewContextClass = flat_review.Context
 
         self.findCommandRun = False
+        self._lastCommandWasStructNav = False
 
         debug.println(debug.LEVEL_FINE, "NEW SCRIPT: %s" % self.name)
 
@@ -181,15 +183,6 @@ class Script:
         """
         return {}
 
-    def getPronunciations(self):
-        """Defines the application specific pronunciations for this script.
-
-        Returns a dictionary where the keys are the actual text strings and
-        the values are the replacement strings that are spoken instead.
-        """
-
-        return {}
-
     def getBrailleCommandsForInputHandler(self, inputEventHandler):
         """Returns a list of BrlTTY commands (they're in braille.py) that
         match the given inputEventHandler passed as argument.
@@ -228,6 +221,14 @@ class Script:
         """
         return None
 
+    def getSpellCheck(self):
+        """Returns the spellcheck support for this script."""
+        return None
+
+    def getCaretNavigation(self):
+        """Returns the caret navigation support for this script."""
+        return None
+
     def getUtilities(self):
         """Returns the utilites for this script.
         """
@@ -244,10 +245,14 @@ class Script:
         return []
 
     def getStructuralNavigation(self):
-        """Returns the 'structural navigation' class for this script.
-        """
+        """Returns the 'structural navigation' class for this script."""
         types = self.getEnabledStructuralNavigationTypes()
-        return structural_navigation.StructuralNavigation(self, types)
+        enable = _settingsManager.getSetting('structuralNavigationEnabled')
+        return structural_navigation.StructuralNavigation(self, types, enable)
+
+    def getLiveRegionManager(self):
+        """Returns the live region support for this script."""
+        return None
 
     def useStructuralNavigationModel(self):
         """Returns True if we should use structural navigation. Most
@@ -278,48 +283,10 @@ class Script:
         """
         return None
 
-    def setAppPreferences(self, prefs):
-        """Write out the application specific preferences lines and set the
-        new values.
+    def getPreferencesFromGUI(self):
+        """Returns a dictionary with the app-specific preferences."""
 
-        Arguments:
-        - prefs: file handle for application preferences.
-        """
-        pass
-
-    def overrideAppKeyBindings(self, script, keyBindings):
-        """Allow for the customization of application specific key bindings.
-
-        Arguments:
-        - script: the application script.
-        - keyBindings: the set of key bindings for this script.
-        """
-
-        return keyBindings
-
-    def overridePronunciations(self, script, pronunciations):
-        """Allow for the customization of application specific pronunciations.
-
-        Arguments:
-        - script: the application script.
-        - pronunciations: the dictionary of pronunciations for this script.
-        """
-
-        return pronunciations
-
-    def getAppState(self):
-        """Returns an object that can be passed to setAppState.  This
-        object will be used by setAppState to restore any state
-        information that was being maintained by the script."""
-        return None
-
-    def setAppState(self, appState):
-        """Sets the application state using the given appState object.
-
-        Arguments:
-        - appState: an object obtained from getAppState
-        """
-        return
+        return {}
 
     def registerEventListeners(self):
         """Tells the event manager to start listening for all the event types
@@ -341,26 +308,6 @@ class Script:
 
         _eventManager.deregisterScriptListeners(self)
 
-    def getSettings(self):
-        """Returns the settings associated with this script, regardless of
-        whether or not the script is active.
-        """
-
-        scriptSettings = settings
-        if orca_state.activeScript != self:
-            name = _scriptManager.getModuleName(self.app)
-            if name:
-                for package in _settingsManager.settingsPackages:
-                    name = package + "." + name
-                    try:
-                        module = importlib.import_module(name)
-                        imp.reload(module)
-                        scriptSettings = module.orca.settings
-                    except:
-                        pass
-
-        return scriptSettings
-
     # [[[WDW - There is a circular reference going on somewhere (see
     # bug 333168).  In the presence of this reference, the existence
     # of a __del__ method prevents the garbage collector from
@@ -437,7 +384,7 @@ class Script:
         if not cachedEvent or cachedEvent == event:
             return False
 
-        focus    = ["focus:", "object:state-changed:focused"]
+        focus    = ["object:state-changed:focused"]
         typing   = ["object:text-changed:insert", "object:text-changed:delete"]
         arrowing = ["object:text-caret-moved", "object:text-selection-changed",
                     "object:selection-changed", "object:active-descendant-changed"]
@@ -452,12 +399,23 @@ class Script:
              and event.detail1 == cachedEvent.detail1:
             skip = True
             reason = "more recent event of the same type in a different object"
+        elif event.type.endswith("system") and event.source == cachedEvent.source:
+            skip = True
+            reason = "more recent system event in the same object"
+        elif event.type.startswith("object:state-changed") \
+             and event.type == cachedEvent.type \
+             and event.source == cachedEvent.source \
+             and event.detail1 == cachedEvent.detail1:
+            skip = True
+            reason = "appears to be duplicate state-changed event"
 
         if skip:
             debug.println(debug.LEVEL_FINE,
-                          "script.skipObjectEvent: skipped due to %s" \
+                          "script.skipObjectEvent: skipped due to %s:" \
                           % reason)
-
+            debug.println(debug.LEVEL_FINE,
+                          "\tType: %s\n\tSource: %s\n\tDetail1: %s"
+                          % (cachedEvent.type, cachedEvent.source, cachedEvent.detail1))
         return skip
 
     def checkKeyboardEventData(self, keyboardEvent):
@@ -498,18 +456,23 @@ class Script:
             user_bindings = user_bindings_map["default"]
 
         consumes = False
+        self._lastCommandWasStructNav = False
         if user_bindings:
             handler = user_bindings.getInputHandler(keyboardEvent)
             if handler \
                  and handler.function in self.structuralNavigation.functions:
-                return self.useStructuralNavigationModel()
+                consumes = self.useStructuralNavigationModel()
+                if consumes:
+                    self._lastCommandWasStructNav = True
             else:
                 consumes = handler != None
         if not consumes:
             handler = self.keyBindings.getInputHandler(keyboardEvent)
             if handler \
                  and handler.function in self.structuralNavigation.functions:
-                return self.useStructuralNavigationModel()
+                consumes = self.useStructuralNavigationModel()
+                if consumes:
+                    self._lastCommandWasStructNav = True
             else:
                 consumes = handler != None
         return consumes
