@@ -36,12 +36,9 @@ import orca.settings_manager as settings_manager
 import orca.orca_state as orca_state
 import orca.speech as speech
 import orca.scripts.toolkits.Gecko as Gecko
-from orca.orca_i18n import _
 
-from .formatting import Formatting
 from .speech_generator import SpeechGenerator
 from .spellcheck import SpellCheck
-from .script_utilities import Utilities
 
 _settingsManager = settings_manager.getManager()
 
@@ -79,10 +76,6 @@ class Script(Gecko.Script):
                 Script.togglePresentationMode,
                 cmdnames.TOGGLE_PRESENTATION_MODE)
 
-    def getFormatting(self):
-        """Returns the formatting strings for this script."""
-        return Formatting(self)
-
     def getSpeechGenerator(self):
         """Returns the speech generator for this script."""
 
@@ -93,18 +86,13 @@ class Script(Gecko.Script):
 
         return SpellCheck(self)
 
-    def getUtilities(self):
-        """Returns the utilites for this script."""
-
-        return Utilities(self)
-
     def getAppPreferencesGUI(self):
         """Return a GtkGrid containing the application unique configuration
         GUI items for the current application."""
 
         grid = Gecko.Script.getAppPreferencesGUI(self)
 
-        self.sayAllOnLoadCheckButton.set_active(
+        self._sayAllOnLoadCheckButton.set_active(
             _settingsManager.getSetting('sayAllOnLoad'))
 
         spellcheck = self.spellcheck.getAppPreferencesGUI()
@@ -117,7 +105,7 @@ class Script(Gecko.Script):
         """Returns a dictionary with the app-specific preferences."""
 
         prefs = Gecko.Script.getPreferencesFromGUI(self)
-        prefs['sayAllOnLoad'] = self.sayAllOnLoadCheckButton.get_active()
+        prefs['sayAllOnLoad'] = self._sayAllOnLoadCheckButton.get_active()
         prefs.update(self.spellcheck.getPreferencesFromGUI())
 
         return prefs
@@ -127,9 +115,19 @@ class Script(Gecko.Script):
 
         if self.spellcheck.isActive():
             self.spellcheck.presentErrorDetails(not basicOnly)
+
+        super().doWhereAmI(inputEvent, basicOnly)
+
+    def locusOfFocusChanged(self, event, oldFocus, newFocus):
+        """Handles changes of focus of interest to the script."""
+
+        if self.spellcheck.isSuggestionsItem(newFocus):
+            includeLabel = not self.spellcheck.isSuggestionsItem(oldFocus)
+            self.updateBraille(newFocus)
+            self.spellcheck.presentSuggestionListItem(includeLabel=includeLabel)
             return
 
-        Gecko.Script.doWhereAmI(self,inputEvent, basicOnly)
+        super().locusOfFocusChanged(event, oldFocus, newFocus)
 
     def _useFocusMode(self, obj):
         if self.isEditableMessage(obj):
@@ -165,14 +163,7 @@ class Script(Gecko.Script):
             orca.setLocusOfFocus(event, event.source, False)
             self.updateBraille(orca_state.locusOfFocus)
 
-        if self.spellcheck.isSuggestionsItem(event.source) \
-           and self.spellcheck.isSuggestionsItem(orca_state.locusOfFocus):
-            orca.setLocusOfFocus(event, event.source, False)
-            self.updateBraille(orca_state.locusOfFocus)
-            self.spellcheck.presentSuggestionListItem()
-            return
-
-        if not self.inDocumentContent(obj):
+        if not self.utilities.inDocumentContent(obj):
             default.Script.onFocusedChanged(self, event)
             return
 
@@ -180,23 +171,23 @@ class Script(Gecko.Script):
             default.Script.onFocusedChanged(self, event)
             return
 
-        role = obj.getRole()
-        if role != pyatspi.ROLE_DOCUMENT_FRAME:
-            Gecko.Script.onFocusedChanged(self, event)
-            return
-
-        contextObj, contextOffset = self.getCaretContext()
-        if contextObj:
-            return
-
-        orca.setLocusOfFocus(event, obj, notifyScript=False)
+        Gecko.Script.onFocusedChanged(self, event)
 
     def onBusyChanged(self, event):
         """Callback for object:state-changed:busy accessibility events."""
 
         obj = event.source
         if obj.getRole() == pyatspi.ROLE_DOCUMENT_FRAME and not event.detail1:
-            if self.inDocumentContent():
+            try:
+                role = orca_state.locusOfFocus.getRole()
+                name = orca_state.locusOfFocus.name
+            except:
+                pass
+            else:
+                if role in [pyatspi.ROLE_FRAME, pyatspi.ROLE_PAGE_TAB] and name:
+                    orca.setLocusOfFocus(event, event.source, False)
+
+            if self.utilities.inDocumentContent():
                 self.speakMessage(obj.name)
                 self._presentMessage(obj)
 
@@ -359,19 +350,20 @@ class Script(Gecko.Script):
             lastKey, mods = self.utilities.lastKeyAndModifiers()
             if lastKey == "Delete":
                 speech.speak(obj.name)
-                [obj, offset] = self.findFirstCaretContext(obj, 0)
-                self.setCaretPosition(obj, offset)
+                [obj, offset] = self.utilities.findFirstCaretContext(obj, 0)
+                self.utilities.setCaretPosition(obj, offset)
                 return
 
     def _presentMessage(self, documentFrame):
         """Presents the first line of the message, or the entire message,
         depending on the user's sayAllOnLoad setting."""
 
-        [obj, offset] = self.findFirstCaretContext(documentFrame, 0)
-        self.setCaretPosition(obj, offset)
+        [obj, offset] = self.utilities.findFirstCaretContext(documentFrame, 0)
+        self.utilities.setCaretPosition(obj, offset)
         self.updateBraille(obj)
         if not _settingsManager.getSetting('sayAllOnLoad'):
-            self.presentLine(obj, offset)
+            contents = self.utilities.getLineContentsAtOffset(obj, offset)
+            self.speakContents(contents)
         elif _settingsManager.getSetting('enableSpeech'):
             self.sayAll(None)
 
@@ -386,21 +378,18 @@ class Script(Gecko.Script):
 
         Gecko.Script.sayCharacter(self, obj)
 
-    def getBottomOfFile(self):
-        """Returns the object and last caret offset at the bottom of the
-        document frame. Overridden here to handle editable messages.
-        """
+    def sayWord(self, obj):
+        """Speaks the word at the current caret position."""
 
-        # Pylint thinks that obj is an instance of a list. It most
-        # certainly is not. Silly pylint.
-        #
-        # pylint: disable-msg=E1103
-        #
-        [obj, offset] = Gecko.Script.getBottomOfFile(self)
-        if obj and obj.getState().contains(pyatspi.STATE_EDITABLE):
-            offset += 1
+        contextObj, offset = self.utilities.getCaretContext(documentFrame=None)
+        if contextObj != obj:
+            Gecko.Script.sayWord(self, obj)
+            return
 
-        return [obj, offset]
+        wordContents = self.utilities.getWordContentsAtOffset(obj, offset)
+        textObj, startOffset, endOffset, word = wordContents[0]
+        self.speakMisspelledIndicator(textObj, startOffset)
+        self.speakContents(wordContents)
 
     def toggleFlatReviewMode(self, inputEvent=None):
         """Toggles between flat review mode and focus tracking mode."""
