@@ -67,9 +67,19 @@ class Script(gtk.Script):
 
         if self.spellcheck.isActive():
             self.spellcheck.presentErrorDetails(not basicOnly)
+
+        super().doWhereAmI(inputEvent, basicOnly)
+
+    def locusOfFocusChanged(self, event, oldFocus, newFocus):
+        """Handles changes of focus of interest to the script."""
+
+        if self.spellcheck.isSuggestionsItem(newFocus):
+            includeLabel = not self.spellcheck.isSuggestionsItem(oldFocus)
+            self.updateBraille(newFocus)
+            self.spellcheck.presentSuggestionListItem(includeLabel=includeLabel)
             return
 
-        gtk.Script.doWhereAmI(self,inputEvent, basicOnly)
+        super().locusOfFocusChanged(event, oldFocus, newFocus)
 
     def onActiveDescendantChanged(self, event):
         """Callback for object:active-descendant-changed accessibility events."""
@@ -94,13 +104,6 @@ class Script(gtk.Script):
         if not event.detail1:
             return
 
-        if self.spellcheck.isSuggestionsItem(event.source) \
-           and self.spellcheck.isSuggestionsItem(orca_state.locusOfFocus):
-            orca.setLocusOfFocus(event, event.source, False)
-            self.updateBraille(orca_state.locusOfFocus)
-            self.spellcheck.presentSuggestionListItem()
-            return
-
         gtk.Script.onFocusedChanged(self, event)
 
     def onNameChanged(self, event):
