@@ -31,7 +31,9 @@ import pyatspi
 import re
 
 import orca.script_utilities as script_utilities
-import orca.settings as settings
+import orca.keybindings as keybindings
+import orca.orca as orca
+import orca.orca_state as orca_state
 
 #############################################################################
 #                                                                           #
@@ -47,28 +49,7 @@ class Utilities(script_utilities.Utilities):
         Arguments:
         - script: the script with which this instance is associated.
         """
-
-        script_utilities.Utilities.__init__(self, script)
-
-    def headingLevel(self, obj):
-        """Determines the heading level of the given object.  A value
-        of 0 means there is no heading level."""
-
-        level = 0
-
-        if obj is None:
-            return level
-
-        if obj.getRole() == pyatspi.ROLE_HEADING:
-            attributes = obj.getAttributes()
-            if attributes is None:
-                return level
-            for attribute in attributes:
-                if attribute.startswith("level:"):
-                    level = int(attribute.split(":")[1])
-                    break
-
-        return level
+        super().__init__(script)
 
     def isWebKitGtk(self, obj):
         """Returns True if this object is a WebKitGtk object."""
@@ -76,9 +57,31 @@ class Utilities(script_utilities.Utilities):
         if not obj:
             return False
 
-        attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            return False
         return attrs.get('toolkit', '') == 'WebKitGtk'
 
+    def getCaretContext(self):
+        # TODO - JD: This is private, but it's only here temporarily until we
+        # have the shared web content support.
+        obj, offset = self._script._lastCaretContext
+        if not obj and self.isWebKitGtk(orca_state.locusOfFocus):
+            obj, offset = super().getCaretContext()
+
+        return obj, offset
+
+    def setCaretContext(self, obj, offset):
+        # TODO - JD: This is private, but it's only here temporarily until we
+        # have the shared web content support.
+        self._script._lastCaretContext = obj, offset
+        orca.setLocusOfFocus(None, obj, notifyScript=False)
+
+    def setCaretPosition(self, obj, offset):
+        self.setCaretContext(obj, offset)
+        self.setCaretOffset(obj, offset)
+
     def isReadOnlyTextArea(self, obj):
         """Returns True if obj is a text entry area that is read only."""
 
@@ -112,6 +115,13 @@ class Utilities(script_utilities.Utilities):
 
         return text
 
+    def getLineContentsAtOffset(self, obj, offset, layoutMode=True, useCache=True):
+        return self.getObjectsFromEOCs(
+            obj, offset, pyatspi.TEXT_BOUNDARY_LINE_START)
+
+    def getObjectContentsAtOffset(self, obj, offset=0, useCache=True):
+        return self.getObjectsFromEOCs(obj, offset)
+
     def getObjectsFromEOCs(self, obj, offset=None, boundary=None):
         """Breaks the string containing a mixture of text and embedded object
         characters into a list of (obj, startOffset, endOffset, string) tuples.
@@ -142,7 +152,7 @@ class Utilities(script_utilities.Utilities):
         else:
             if boundary == pyatspi.TEXT_BOUNDARY_CHAR:
                 key, mods = self.lastKeyAndModifiers()
-                if (mods & settings.SHIFT_MODIFIER_MASK) and key == 'Right':
+                if (mods & keybindings.SHIFT_MODIFIER_MASK) and key == 'Right':
                     offset -= 1
             segment, start, end = text.getTextAtOffset(offset, boundary)
 
@@ -245,3 +255,49 @@ class Utilities(script_utilities.Utilities):
             return self.onSameLine(obj[0], obj[1])
 
         return False
+
+    def isEmbeddedDocument(self, obj):
+        if not self.isWebKitGtk(obj):
+            return False
+
+        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
+        if not (obj and obj.getRole() in docRoles):
+            return False
+
+        parent = obj.parent
+        if not (parent and self.isWebKitGtk(parent)):
+            return False
+
+        parent = parent.parent
+        if not (parent and not self.isWebKitGtk(parent)):
+            return False
+
+        return True
+
+    def setCaretAtStart(self, obj):
+        def implementsText(obj):
+            if obj.getRole() == pyatspi.ROLE_LIST:
+                return False
+            return 'Text' in pyatspi.utils.listInterfaces(obj)
+
+        child = obj
+        if not implementsText(obj):
+            child = pyatspi.utils.findDescendant(obj, implementsText)
+            if not child:
+                return None, -1
+
+        index = -1
+        text = child.queryText()
+        for i in range(text.characterCount):
+            if text.setCaretOffset(i):
+                index = i
+                break
+
+        return child, index
+
+    def treatAsBrowser(self, obj):
+        return self.isEmbeddedDocument(obj)
+
+    def inDocumentContent(self, obj=None):
+        obj = obj or orca_state.locusOfFocus
+        return self.isWebKitGtk(obj)
