@@ -57,9 +57,8 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return voice
 
     def _generateLabel(self, obj, **args):
-        result = \
-            speech_generator.SpeechGenerator._generateLabel(self, obj, **args)
-        if result:
+        result = super()._generateLabel(obj, **args)
+        if result or not self._script.utilities.isWebKitGtk(obj):
             return result
 
         role = args.get('role', obj.getRole())
@@ -72,7 +71,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not role in inferRoles:
             return result
 
-        label = self._script.labelInference.infer(obj)
+        label, objects = self._script.labelInference.infer(obj)
         if label:
             result.append(label)
             result.extend(self.voice(speech_generator.DEFAULT))
@@ -112,9 +111,10 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                                pyatspi.ROLE_TABLE_CELL])
 
         if not (role in doNotSpeak):
+            docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
             if role == pyatspi.ROLE_IMAGE:
                 link = self._script.utilities.ancestorWithRole(
-                    obj, [pyatspi.ROLE_LINK], [pyatspi.ROLE_DOCUMENT_FRAME])
+                    obj, [pyatspi.ROLE_LINK], docRoles)
                 if link:
                     result.append(self.getLocalizedRoleName(link))
             elif role == pyatspi.ROLE_HEADING:
@@ -154,6 +154,17 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if role == pyatspi.ROLE_LINK:
             return []
 
+        args['stopAtRoles'] = [pyatspi.ROLE_DOCUMENT_FRAME,
+                               pyatspi.ROLE_DOCUMENT_WEB,
+                               pyatspi.ROLE_EMBEDDED,
+                               pyatspi.ROLE_INTERNAL_FRAME,
+                               pyatspi.ROLE_FORM,
+                               pyatspi.ROLE_MENU_BAR,
+                               pyatspi.ROLE_TOOL_BAR]
+        args['skipRoles'] = [pyatspi.ROLE_PARAGRAPH,
+                             pyatspi.ROLE_LIST_ITEM,
+                             pyatspi.ROLE_TEXT]
+
         return speech_generator.SpeechGenerator._generateAncestors(
             self, obj, **args)
 
