@@ -29,13 +29,13 @@ import pyatspi
 import urllib.parse, urllib.request, urllib.error, urllib.parse
 from gi.repository import Atspi, Atk
 
+from . import chnames
 from . import debug
 from . import generator
 from . import messages
 from . import object_properties
 from . import settings
 from . import settings_manager
-from . import sound
 from . import text_attribute_names
 from . import acss
 
@@ -93,28 +93,23 @@ class SpeechGenerator(generator.Generator):
     def __init__(self, script):
         generator.Generator.__init__(self, script, "speech")
 
+    def _getACSS(self, obj, string):
+        if obj.getRole() == pyatspi.ROLE_LINK:
+            acss = self.voice(HYPERLINK)
+        elif isinstance(string, str) \
+            and string.isupper() \
+            and string.strip().isalpha():
+            acss = self.voice(UPPERCASE)
+        else:
+            acss = self.voice(DEFAULT)
+
+        return acss
+
     def _addGlobals(self, globalsDict):
         """Other things to make available from the formatting string.
         """
         generator.Generator._addGlobals(self, globalsDict)
         globalsDict['voice'] = self.voice
-        globalsDict['play'] = self.play
-
-    def play(self, key):
-        """Returns an array containing a sound.Sound instance.
-        The key can a value to be used to look up a filename in the
-        settings.py:sounds dictionary (e.g., a pyatspi.ROLE_* value)
-        or just the name of an audio file to use.
-        """
-        sounds = _settingsManager.getSetting('sounds')
-        try:
-            soundBite = sound.Sound(sounds[key])
-        except:
-            if isinstance(key, str):
-                soundBite = sound.Sound(key)
-            else:
-                soundBite = None
-        return [soundBite]
 
     def generateSpeech(self, obj, **args):
         return self.generate(obj, **args)
@@ -179,9 +174,18 @@ class SpeechGenerator(generator.Generator):
         acss = self.voice(DEFAULT)
         result.extend(self._generateLabel(obj, **args))
         if not result:
-            if obj.name and (len(obj.name)):
-                result.append(obj.name)
+            try:
+                name = obj.name
+            except:
+                msg = 'ERROR: Could not get name for %s' % obj
+                debug.println(debug.LEVEL_INFO, msg)
+                return result
+            if name:
+                result.append(name)
                 result.extend(acss)
+        if not result and obj.parent.getRole() == pyatspi.ROLE_AUTOCOMPLETE:
+            result = self._generateLabelOrName(obj.parent, **args)
+
         return result
 
     def _generatePlaceholderText(self, obj, **args):
@@ -265,6 +269,20 @@ class SpeechGenerator(generator.Generator):
             result.extend(acss)
         return result
 
+    def _generateTable(self, obj, **args):
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        if _settingsManager.getSetting('speechVerbosityLevel') == \
+           settings.VERBOSITY_LEVEL_BRIEF:
+            return []
+
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateTable(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
     def _generateTextRole(self, obj, **args):
         """A convenience method to prevent the pyatspi.ROLE_PARAGRAPH role
         from being spoken. In the case of a pyatspi.ROLE_PARAGRAPH
@@ -296,6 +314,9 @@ class SpeechGenerator(generator.Generator):
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
 
+        if self._script.utilities.isStatusBarNotification(obj):
+            return []
+
         result = []
         acss = self.voice(SYSTEM)
         role = args.get('role', obj.getRole())
@@ -304,15 +325,31 @@ class SpeechGenerator(generator.Generator):
                         pyatspi.ROLE_FILLER,
                         pyatspi.ROLE_EXTENDED]
 
+        try:
+            parentRole = obj.parent.getRole()
+        except:
+            parentRole = None
+
+        if role == pyatspi.ROLE_MENU and parentRole == pyatspi.ROLE_COMBO_BOX:
+            return self._generateRoleName(obj.parent)
+
         # egg-list-box, e.g. privacy panel in gnome-control-center
-        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
+        if parentRole == pyatspi.ROLE_LIST_BOX:
             doNotPresent.append(obj.getRole())
 
         if _settingsManager.getSetting('speechVerbosityLevel') \
                 == settings.VERBOSITY_LEVEL_BRIEF:
             doNotPresent.extend([pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS])
 
-        if role not in doNotPresent:
+        if role == pyatspi.ROLE_HEADING:
+            level = self._script.utilities.headingLevel(obj)
+            if level:
+                result.append(object_properties.ROLE_HEADING_LEVEL_SPEECH % {
+                    'role': self.getLocalizedRoleName(obj, role),
+                    'level': level})
+                result.extend(acss)
+
+        if role not in doNotPresent and not result:
             result.append(self.getLocalizedRoleName(obj, role))
             result.extend(acss)
         return result
@@ -327,8 +364,7 @@ class SpeechGenerator(generator.Generator):
         """
         return self._generateRoleName(obj, **args)
 
-    @staticmethod
-    def getLocalizedRoleName(obj, role=None):
+    def getLocalizedRoleName(self, obj, role=None):
         """Returns the localized name of the given Accessible object; the name
         is suitable to be spoken.
 
@@ -359,7 +395,8 @@ class SpeechGenerator(generator.Generator):
         """
         result = []
         acss = self.voice(DEFAULT)
-        labels = self._script.utilities.unrelatedLabels(obj)
+        visibleOnly = not self._script.utilities.isStatusBarNotification(obj)
+        labels = self._script.utilities.unrelatedLabels(obj, visibleOnly)
         for label in labels:
             name = self._generateName(label, **args)
             result.extend(name)
@@ -367,23 +404,6 @@ class SpeechGenerator(generator.Generator):
             result.extend(acss)
         return result
 
-    def _generateEmbedded(self, obj, **args):
-        """Returns an array of strings (and possibly voice and audio
-        specifications) used especially for handling embedded objects.
-        This either is the label or name of the object or the name of
-        the application for the object.
-        """
-        acss = self.voice(DEFAULT)
-        result = self._generateLabelOrName(obj, **args)
-        if not result:
-            try:
-                result.append(obj.getApplication().name)
-            except:
-                pass
-        if result:
-            result.extend(acss)
-        return result
-
     #####################################################################
     #                                                                   #
     # State information                                                 #
@@ -446,10 +466,8 @@ class SpeechGenerator(generator.Generator):
 
         result = []
         acss = self.voice(STATE)
-        if obj.getState().contains(pyatspi.STATE_MULTISELECTABLE):
-            # Translators: "multi-select" refers to a web form list
-            # in which more than one item can be selected at a time.
-            #
+        if obj.getState().contains(pyatspi.STATE_MULTISELECTABLE) \
+           and obj.childCount:
             result.append(self._script.formatting.getString(
                 mode='speech',
                 stringType='multiselect'))
@@ -765,41 +783,31 @@ class SpeechGenerator(generator.Generator):
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
 
-        result = []
-        acss = self.voice(STATE)
-        # If this is an icon within an layered pane or a table cell
-        # within a table or a tree table and the item is focused but not
-        # selected, let the user know. See bug #486908 for more details.
-        #
-        checkIfSelected = False
-        objRole, parentRole, state = None, None, None
-        if obj:
-            objRole = obj.getRole()
-            state = obj.getState()
-            if obj.parent:
-                parentRole = obj.parent.getRole()
+        if not obj:
+            return []
 
-        if objRole == pyatspi.ROLE_TABLE_CELL \
-           and (parentRole == pyatspi.ROLE_TREE_TABLE \
-                or parentRole == pyatspi.ROLE_TABLE):
-            checkIfSelected = True
+        if not (obj.parent and 'Selection' in pyatspi.listInterfaces(obj.parent)):
+            return []
 
-        # If we met the last set of conditions, but we got here by
-        # moving left or right on the same row, then don't announce the
-        # selection state to the user. See bug #523235 for more details.
-        #
-        lastKey, mods = self._script.utilities.lastKeyAndModifiers()
-        if checkIfSelected and lastKey in ["Left", "Right"]:
-            checkIfSelected = False
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_SELECTED):
+            return []
 
-        if objRole == pyatspi.ROLE_ICON \
-           and parentRole == pyatspi.ROLE_LAYERED_PANE:
-            checkIfSelected = True
+        parentRole = obj.parent.getRole()
+        if parentRole in [pyatspi.ROLE_TREE_TABLE, pyatspi.ROLE_TABLE]:
+            lastKey, mods = self._script.utilities.lastKeyAndModifiers()
+            if lastKey in ["Left", "Right"]:
+                return []
+            if self._script.utilities.isLayoutOnly(obj.parent):
+                return []
+        elif parentRole == pyatspi.ROLE_LAYERED_PANE:
+            if obj in self._script.utilities.selectedChildren(obj.parent):
+                return []
+        else:
+            return []
 
-        if checkIfSelected \
-           and state and not state.contains(pyatspi.STATE_SELECTED):
-            result.append(object_properties.STATE_UNSELECTED_TABLE_CELL)
-            result.extend(acss)
+        result = [object_properties.STATE_UNSELECTED_TABLE_CELL]
+        result.extend(self.voice(STATE))
 
         return result
 
@@ -959,6 +967,11 @@ class SpeechGenerator(generator.Generator):
         this is a text object.  [[[WDW - consider returning an empty
         array if this is not a text object.]]]
         """
+
+        result = self._generateSubstring(obj, **args)
+        if result:
+            return result
+
         acss = self.voice(DEFAULT)
         result = generator.Generator._generateCurrentLineText(self, obj, **args)
         if result:
@@ -968,9 +981,19 @@ class SpeechGenerator(generator.Generator):
         return result
 
     def _generateDisplayedText(self, obj, **args):
+        result = self._generateSubstring(obj, **args)
+        if result:
+            return result
+
         acss = self.voice(DEFAULT)
         result = generator.Generator._generateDisplayedText(self, obj, **args)
         if result:
+            string = result[0].strip()
+            if len(string) == 1 and self._script.utilities.isMath(obj):
+                charname = chnames.getCharacterName(string)
+                if charname != string:
+                    result[0] = charname
+
             result.extend(acss)
 
         return result
@@ -1049,35 +1072,17 @@ class SpeechGenerator(generator.Generator):
 
         textObj = obj.queryText()
         caretOffset = textObj.caretOffset
-        textContents = ""
-        selected = False
 
-        nSelections = textObj.getNSelections()
+        textContents, startOffset, endOffset = self._script.utilities.allSelectedText(obj)
+        selected = textContents != ""
 
-        [current, other] = self._script.utilities.hasTextSelections(obj)
-        if current or other:
-            selected = True
-            [textContents, startOffset, endOffset] = \
-                self._script.utilities.allSelectedText(obj)
-        else:
+        if not selected:
             # Get the line containing the caret
             #
             [line, startOffset, endOffset] = textObj.getTextAtOffset(
                 textObj.caretOffset,
                 pyatspi.TEXT_BOUNDARY_LINE_START)
             if len(line):
-                # Check for embedded object characters. If we find any,
-                # expand the text. TODO - JD: This expansion doesn't
-                # include the role information; just the text. However,
-                # the handling of roles should probably be dealt with as
-                # a formatting string. We have not yet worked out how to
-                # do this with Gecko (primary user of embedded object
-                # characters). Until we do, this expansion is better than
-                # presenting the actual embedded object character.
-                #
-                if self._script.EMBEDDED_OBJECT_CHARACTER in line:
-                    line = self._script.utilities.expandEOCs(
-                        obj, startOffset, endOffset)
                 line = self._script.utilities.adjustForRepeats(line)
                 textContents = line
             else:
@@ -1097,6 +1102,11 @@ class SpeechGenerator(generator.Generator):
         specifications) containing the text content.  This requires
         _generateTextInformation to have been called prior to this method.
         """
+
+        result = self._generateSubstring(obj, **args)
+        if result:
+            return result
+
         try:
             text = obj.queryText()
         except NotImplementedError:
@@ -1212,6 +1222,17 @@ class SpeechGenerator(generator.Generator):
                     result.extend(acss)
         return result
 
+    def _generateSubstring(self, obj, **args):
+        result = super()._generateSubstring(obj, **args)
+        if not (result and result[0]):
+            return []
+
+        if not obj.getState().contains(pyatspi.STATE_EDITABLE):
+            result[0] = result[0].strip()
+
+        result.extend(self._getACSS(obj, result[0]))
+        return result
+
     def generateTextIndentation(self, obj, **args):
         return self._generateTextIndentation(obj, **args)
 
@@ -1431,16 +1452,10 @@ class SpeechGenerator(generator.Generator):
         if role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]:
             return result
 
-        try:
-            s = obj.querySelection()
-        except NotImplementedError:
-            isFocused = \
-                lambda x: x and x.getState().contains(pyatspi.STATE_FOCUSED)
-            items = pyatspi.utils.findAllDescendants(obj, isFocused)
-        else:
-            items = [s.getSelectedChild(i) for i in range(s.nSelectedChildren)]
-            if not items and obj.childCount:
-                items.append(obj[0])
+        items = self._script.utilities.selectedChildren(obj)
+        items = items or [self._script.utilities.focusedChild(obj)]
+        if not (items and items[0]):
+            return result
 
         items = list(map(self._generateName, items))
         for item in items:
@@ -1579,6 +1594,14 @@ class SpeechGenerator(generator.Generator):
         'priorObj' is typically set by Orca to be the previous object
         with focus.
         """
+
+        # TODO - JD: This is not the right way to do this, but we can fix
+        # that as part of the removal of formatting strings.
+        start = args.get('startOffset')
+        end = args.get('endOffset')
+        if start is not None or end is not None:
+            return []
+
         result = []
         priorObj = args.get('priorObj')
         if obj.getRole() == pyatspi.ROLE_MENU_ITEM \
@@ -1599,7 +1622,7 @@ class SpeechGenerator(generator.Generator):
            and args.get('formatType', None) \
                in ['basicWhereAmI', 'detailedWhereAmI']:
             return [object_properties.ROLE_ICON_PANEL]
-        elif obj.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
+        if obj.parent.getRole() in [pyatspi.ROLE_TABLE_CELL, pyatspi.ROLE_MENU]:
             obj = obj.parent
         return self._generateRoleName(obj.parent)
 
@@ -1660,87 +1683,29 @@ class SpeechGenerator(generator.Generator):
         specifications) that represent the relative position of an
         object in a list.
         """
+
         if _settingsManager.getSetting('onlySpeakDisplayedText') \
            or not (_settingsManager.getSetting('enablePositionSpeaking') \
                    or args.get('forceList', False)):
             return []
 
+        if obj.getRole() == pyatspi.ROLE_MENU \
+           and obj.parent == self._script.utilities.topLevelObject(obj):
+            return []
+
         result = []
         acss = self.voice(SYSTEM)
-        position = -1
-        index = 0
-        total = 0
-        name = self._generateName(obj)
-        # TODO - JD: There might be a better way to do this (e.g. pass
-        # roles in maybe?).
-        #
-        role = args.get('role', obj.getRole())
-        if role == pyatspi.ROLE_COMBO_BOX:
-            obj = obj[0]
-        elif role in [pyatspi.ROLE_PAGE_TAB,
-                      pyatspi.ROLE_MENU,
-                      pyatspi.ROLE_MENU_ITEM,
-                      pyatspi.ROLE_CHECK_MENU_ITEM,
-                      pyatspi.ROLE_RADIO_MENU_ITEM]:
-            obj = obj.parent
-        elif role == pyatspi.ROLE_LIST_ITEM:
-            parent = obj.parent
-            for relation in obj.getRelationSet():
-                if relation.getRelationType() == \
-                        pyatspi.RELATION_NODE_CHILD_OF:
-                    # childNodes assumes that we have an accessible table
-                    # interface to work with. If we don't, it will fail. So
-                    # don't set the parent until verifying the interface we
-                    # expect actually exists.
-                    #
-                    target = relation.getTarget(0)
-                    try:
-                        target.parent.queryTable()
-                    except:
-                        pass
-                    else:
-                        parent = target
-                    break
-            obj = parent
-        else:
-            obj = obj.parent
+        position, total = self._script.utilities.getPositionAndSetSize(obj)
+        if position < 0 or total < 0:
+            return []
 
-        # We want to return the position relative to this hierarchical
-        # level and not the entire list.  If the object in question
-        # uses the NODE_CHILD_OF relationship, we need to use it instead
-        # of the childCount.
-        #
-        childNodes = self._script.utilities.childNodes(obj)
-        total = len(childNodes)
-        for i in range(0, total):
-            childName = self._generateName(childNodes[i])
-            if childName == name:
-                position = i+1
-                break
-
-        if not total:
-            for child in obj:
-                nextName = self._generateName(child)
-                state = child.getState()
-                if not nextName or nextName[0] in ["", "Empty", "separator"] \
-                   or not state.contains(pyatspi.STATE_VISIBLE):
-                    continue
-
-                index += 1
-                total += 1
-
-                if nextName == name:
-                    position = index
-
-        if (_settingsManager.getSetting('enablePositionSpeaking') \
-             or args.get('forceList', False)) \
-           and position >= 0:
-            result.append(self._script.formatting.getString(
+        position += 1
+        result.append(self._script.formatting.getString(
                               mode='speech',
                               stringType='groupindex') \
                           % {"index" : position,
                              "total" : total})
-            result.extend(acss)
+        result.extend(acss)
         return result
 
     def _generateDefaultButton(self, obj, **args):
@@ -1768,15 +1733,22 @@ class SpeechGenerator(generator.Generator):
         specifications) that represent the status bar of a window.
         This method should initially be called with a top-level window.
         """
-        result = []
+
         statusBar = self._script.utilities.statusBar(obj)
-        if statusBar:
-            name = self._generateName(statusBar)
-            if name:
-                result.extend(name)
-            else:
-                for child in statusBar:
-                    result.extend(self._generateName(child))
+        if not statusBar:
+            return []
+
+        result = self._generateName(statusBar)
+        if result:
+            return result
+
+        for child in statusBar:
+            childResult = self._generateName(child)
+            if not childResult and child.getRole() != pyatspi.ROLE_LABEL:
+                childResult = self.generate(child, includeContext=False)
+            if childResult:
+                result.extend(childResult)
+
         return result
 
     def generateStatusBar(self, obj, **args):
@@ -1876,10 +1848,12 @@ class SpeechGenerator(generator.Generator):
         acss = self.voice(SYSTEM)
         alreadyFocused = args.get('alreadyFocused', False)
         forceTutorial = args.get('forceTutorial', False)
+        role = args.get('role', obj.getRole())
         result.extend(self._script.tutorialGenerator.getTutorial(
                 obj,
                 alreadyFocused,
-                forceTutorial))
+                forceTutorial,
+                role))
         if args.get('role', obj.getRole()) == pyatspi.ROLE_ICON \
             and args.get('formatType', 'unfocused') == 'basicWhereAmI':
             frame, dialog = self._script.utilities.frameAndDialog(obj)
@@ -1892,13 +1866,382 @@ class SpeechGenerator(generator.Generator):
             result.extend(acss)
         return result
 
+    # Math
+
+    def _generateMath(self, obj, **args):
+        result = []
+        children = [child for child in obj] or [obj]
+        for child in children:
+            if self._script.utilities.isMathLayoutOnly(child) and child.childCount:
+                result.extend(self._generateMath(child))
+                continue
+
+            oldRole = self._getAlternativeRole(child)
+            self._overrideRole(oldRole, args)
+            result.extend(self.generate(child, role=oldRole))
+            self._restoreRole(oldRole, args)
+
+        return result
+
+    def _generateEnclosedBase(self, obj, **args):
+        return self._generateMath(obj, **args)
+
+    def _generateEnclosedEnclosures(self, obj, **args):
+        strings = []
+        enclosures = self._script.utilities.getMathEnclosures(obj)
+        if 'actuarial' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_ACTUARIAL)
+        if 'box' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_BOX)
+        if 'circle' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_CIRCLE)
+        if 'longdiv' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_LONGDIV)
+        if 'radical' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_RADICAL)
+        if 'roundedbox' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_ROUNDEDBOX)
+        if 'horizontalstrike' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_HORIZONTALSTRIKE)
+        if 'verticalstrike' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_VERTICALSTRIKE)
+        if 'downdiagonalstrike' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_DOWNDIAGONALSTRIKE)
+        if 'updiagonalstrike' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_UPDIAGONALSTRIKE)
+        if 'northeastarrow' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_NORTHEASTARROW)
+        if 'bottom' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_BOTTOM)
+        if 'left' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_LEFT)
+        if 'right' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_RIGHT)
+        if 'top' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_TOP)
+        if 'phasorangle' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_PHASOR_ANGLE)
+        if 'madruwb' in enclosures:
+            strings.append(messages.MATH_ENCLOSURE_MADRUWB)
+        if not strings:
+            msg = 'INFO: Could not get enclosure message for %s' % enclosures
+            debug.println(debug.LEVEL_INFO, msg)
+            return []
+
+        if len(strings) == 1:
+            result = [messages.MATH_ENCLOSURE_ENCLOSED_BY % strings[0]]
+        else:
+            strings.insert(-1, messages.MATH_ENCLOSURE_AND)
+            if len(strings) == 3:
+                result = [messages.MATH_ENCLOSURE_ENCLOSED_BY % " ".join(strings)]
+            else:
+                result = [messages.MATH_ENCLOSURE_ENCLOSED_BY % ", ".join(strings)]
+
+        result.extend(self.voice(SYSTEM))
+        return result
+
+    def _generateFencedStart(self, obj, **args):
+        fenceStart, fenceEnd = self._script.utilities.getMathFences(obj)
+        if fenceStart:
+            result = [chnames.getCharacterName(fenceStart)]
+            result.extend(self.voice(DEFAULT))
+            return result
+
+        return []
+
+    def _generateFencedContents(self, obj, **args):
+        result = []
+        separators = self._script.utilities.getMathFencedSeparators(obj)
+        for x in range(len(separators), obj.childCount-1):
+            separators.append(separators[-1])
+        separators.append('')
+
+        for i, child in enumerate(obj):
+            result.extend(self._generateMath(child, **args))
+            separatorName = chnames.getCharacterName(separators[i])
+            result.append(separatorName)
+            result.extend(self.voice(DEFAULT))
+            if separatorName:
+                result.extend(self._generatePause(obj, **args))
+
+        return result
+
+    def _generateFencedEnd(self, obj, **args):
+        fenceStart, fenceEnd = self._script.utilities.getMathFences(obj)
+        if fenceEnd:
+            result = [chnames.getCharacterName(fenceEnd)]
+            result.extend(self.voice(DEFAULT))
+            return result
+
+        return []
+
+    def _generateFractionStart(self, obj, **args):
+        if self._script.utilities.isMathFractionWithoutBar(obj):
+            result = [messages.MATH_FRACTION_WITHOUT_BAR_START]
+        else:
+            result = [messages.MATH_FRACTION_START]
+        result.extend(self.voice(SYSTEM))
+        return result
+
+    def _generateFractionNumerator(self, obj, **args):
+        numerator = self._script.utilities.getMathNumerator(obj)
+        if self._script.utilities.isMathLayoutOnly(numerator):
+            return self._generateMath(numerator)
+
+        oldRole = self._getAlternativeRole(numerator)
+        self._overrideRole(oldRole, args)
+        result = self.generate(numerator, role=oldRole)
+        self._restoreRole(oldRole, args)
+        return result
+
+    def _generateFractionDenominator(self, obj, **args):
+        denominator = self._script.utilities.getMathDenominator(obj)
+        if self._script.utilities.isMathLayoutOnly(denominator):
+            return self._generateMath(denominator)
+
+        oldRole = self._getAlternativeRole(denominator)
+        self._overrideRole(oldRole, args)
+        result = self.generate(denominator, role=oldRole)
+        self._restoreRole(oldRole, args)
+        return result
+
+    def _generateFractionLine(self, obj, **args):
+        result = [messages.MATH_FRACTION_LINE]
+        result.extend(self.voice(SYSTEM))
+        return result
+
+    def _generateFractionEnd(self, obj, **args):
+        result = [messages.MATH_FRACTION_END]
+        result.extend(self.voice(SYSTEM))
+        return result
+
+    def _generateRootStart(self, obj, **args):
+        result = []
+        if self._script.utilities.isMathSquareRoot(obj):
+            result = [messages.MATH_SQUARE_ROOT_OF]
+        else:
+            index = self._script.utilities.getMathRootIndex(obj)
+            string = self._script.utilities.displayedText(index)
+            if string == "2":
+                result = [messages.MATH_SQUARE_ROOT_OF]
+            elif string == "3":
+                result = [messages.MATH_CUBE_ROOT_OF]
+            elif string:
+                result = [string]
+                result.extend([messages.MATH_ROOT_OF])
+            elif self._script.utilities.isMathLayoutOnly(index):
+                result = self._generateMath(index)
+                result.extend([messages.MATH_ROOT_OF])
+            else:
+                oldRole = self._getAlternativeRole(index)
+                self._overrideRole(oldRole, args)
+                result.extend(self.generate(index, role=oldRole))
+                self._restoreRole(oldRole, args)
+                result.extend([messages.MATH_ROOT_OF])
+
+        if result:
+            result.extend(self.voice(SYSTEM))
+
+        return result
+
+    def _generateRootBase(self, obj, **args):
+        base = self._script.utilities.getMathRootBase(obj)
+        if not base:
+            return []
+
+        if self._script.utilities.isMathSquareRoot(obj) \
+           or self._script.utilities.isMathToken(base) \
+           or self._script.utilities.isMathLayoutOnly(base):
+            return self._generateMath(base)
+
+        result = [self._generatePause(obj, **args)]
+        oldRole = self._getAlternativeRole(base)
+        self._overrideRole(oldRole, args)
+        result.extend(self.generate(base, role=oldRole))
+        self._restoreRole(oldRole, args)
+
+        return result
+
+    def _generateRootEnd(self, obj, **args):
+        result = [messages.MATH_ROOT_END]
+        result.extend(self.voice(SYSTEM))
+        return result
+
+    def _generateScriptBase(self, obj, **args):
+        base = self._script.utilities.getMathScriptBase(obj)
+        if not base:
+            return []
+
+        return self._generateMath(base)
+
+    def _generateScriptScript(self, obj, **args):
+        if self._script.utilities.isMathLayoutOnly(obj):
+            return self._generateMath(obj)
+
+        oldRole = self._getAlternativeRole(obj)
+        self._overrideRole(oldRole, args)
+        result = self.generate(obj, role=oldRole)
+        self._restoreRole(oldRole, args)
+
+        return result
+
+    def _generateScriptSubscript(self, obj, **args):
+        subscript = self._script.utilities.getMathScriptSubscript(obj)
+        if not subscript:
+            return []
+
+        result = [messages.MATH_SUBSCRIPT]
+        result.extend(self.voice(SYSTEM))
+        result.extend(self._generateScriptScript(subscript))
+
+        return result
+
+    def _generateScriptSuperscript(self, obj, **args):
+        superscript = self._script.utilities.getMathScriptSuperscript(obj)
+        if not superscript:
+            return []
+
+        result = [messages.MATH_SUPERSCRIPT]
+        result.extend(self.voice(SYSTEM))
+        result.extend(self._generateScriptScript(superscript))
+
+        return result
+
+    def _generateScriptUnderscript(self, obj, **args):
+        underscript = self._script.utilities.getMathScriptUnderscript(obj)
+        if not underscript:
+            return []
+
+        result = [messages.MATH_UNDERSCRIPT]
+        result.extend(self.voice(SYSTEM))
+        result.extend(self._generateScriptScript(underscript))
+
+        return result
+
+    def _generateScriptOverscript(self, obj, **args):
+        overscript = self._script.utilities.getMathScriptOverscript(obj)
+        if not overscript:
+            return []
+
+        result = [messages.MATH_OVERSCRIPT]
+        result.extend(self.voice(SYSTEM))
+        result.extend(self._generateScriptScript(overscript))
+
+        return result
+
+    def _generateScriptPrescripts(self, obj, **args):
+        result = []
+        prescripts = self._script.utilities.getMathPrescripts(obj)
+        for i, script in enumerate(prescripts):
+            if self._script.utilities.isNoneElement(script):
+                continue
+            if i % 2:
+                rv = [messages.MATH_PRE_SUPERSCRIPT]
+            else:
+                rv = [messages.MATH_PRE_SUBSCRIPT]
+            rv.extend(self.voice(SYSTEM))
+            rv.extend(self._generateScriptScript(script))
+            result.append(rv)
+
+        return result
+
+    def _generateScriptPostscripts(self, obj, **args):
+        result = []
+        postscripts = self._script.utilities.getMathPostscripts(obj)
+        for i, script in enumerate(postscripts):
+            if self._script.utilities.isNoneElement(script):
+                continue
+            if i % 2:
+                rv = [messages.MATH_SUPERSCRIPT]
+            else:
+                rv = [messages.MATH_SUBSCRIPT]
+            rv.extend(self.voice(SYSTEM))
+            rv.extend(self._generateScriptScript(script))
+            result.append(rv)
+
+        return result
+
+    def _generateMathTableStart(self, obj, **args):
+        try:
+            table = obj.queryTable()
+        except:
+            return []
+
+        nestingLevel = self._script.utilities.getMathNestingLevel(obj)
+        if nestingLevel > 0:
+            result = [messages.mathNestedTableSize(table.nRows, table.nColumns)]
+        else:
+            result = [messages.mathTableSize(table.nRows, table.nColumns)]
+        result.extend(self.voice(SYSTEM))
+        return result
+
+    def _generateMathTableRows(self, obj, **args):
+        result = []
+        for row in obj:
+            oldRole = self._getAlternativeRole(row)
+            self._overrideRole(oldRole, args)
+            result.extend(self.generate(row, role=oldRole))
+            self._restoreRole(oldRole, args)
+
+        return result
+
+    def _generateMathRow(self, obj, **args):
+        result = []
+
+        result.append(messages.TABLE_ROW % (obj.getIndexInParent() + 1))
+        result.extend(self.voice(SYSTEM))
+        result.extend(self._generatePause(obj, **args))
+
+        for child in obj:
+            result.extend(self._generateMath(child))
+            result.extend(self._generatePause(child, **args))
+
+        return result
+
+    def _generateMathTableEnd(self, obj, **args):
+        nestingLevel = self._script.utilities.getMathNestingLevel(obj)
+        if nestingLevel > 0:
+            result = [messages.MATH_NESTED_TABLE_END]
+        else:
+            result = [messages.MATH_TABLE_END]
+        result.extend(self.voice(SYSTEM))
+        return result
+
     #####################################################################
     #                                                                   #
     # Other things for prosody and voice selection                      #
     #                                                                   #
     #####################################################################
 
+    def _getAlternativeRole(self, obj, **args):
+        if self._script.utilities.isMath(obj):
+            if self._script.utilities.isMathFraction(obj):
+                return 'ROLE_MATH_FRACTION'
+            if self._script.utilities.isMathRoot(obj):
+                return 'ROLE_MATH_ROOT'
+            if self._script.utilities.isMathSubOrSuperScript(obj):
+                return 'ROLE_MATH_SCRIPT_SUBSUPER'
+            if self._script.utilities.isMathUnderOrOverScript(obj):
+                return 'ROLE_MATH_SCRIPT_UNDEROVER'
+            if self._script.utilities.isMathMultiScript(obj):
+                return 'ROLE_MATH_MULTISCRIPT'
+            if self._script.utilities.isMathEnclose(obj):
+                return 'ROLE_MATH_ENCLOSED'
+            if self._script.utilities.isMathFenced(obj):
+                return 'ROLE_MATH_FENCED'
+            if self._script.utilities.isMathTable(obj):
+                return 'ROLE_MATH_TABLE'
+            if self._script.utilities.isMathTableRow(obj):
+                return 'ROLE_MATH_TABLE_ROW'
+        if self._script.utilities.isStatic(obj):
+            return 'ROLE_STATIC'
+
+        return args.get('role', obj.getRole())
+
     def _generatePause(self, obj, **args):
+        if not _settingsManager.getSetting('enablePauseBreaks') \
+           or args.get('eliminatePauses', False):
+            return []
         return PAUSE
 
     def _generateLineBreak(self, obj, **args):
